package	jp.co.sds_corp.jdbrowser;

import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.HashMap;

import jp.co.sds_corp.util.db.DbColumn;

public class ColumnInfo
{
	public ColumnInfo() {
	}

	public ColumnInfo( String name, String type, String precision, String scale,
			Boolean key, Boolean nullable, String defValue, String remarks )
	{
		this.name = name;
		this.type = type;
		if ( needPrecision( type )) {
			this.precision = Integer.valueOf( precision ).intValue();
			if ( needScale( type )) {
				if ( scale != null && scale.length() > 0 ) {
					this.scale = Integer.valueOf( scale ).intValue();
				} else {
					this.scale = 0;
				}
			}
		}
		if ( key == null ) {
			this.key = false;
		} else {
			this.key = key.booleanValue();
		}
		if ( nullable == null ) {
			this.nullable = true;
		} else {
			this.nullable = !nullable.booleanValue();
		}
		this.defValue = defValue;
		this.remarks = remarks;
	}

	public ColumnInfo( ResultSetMetaData rsmd, int nCol )
		throws SQLException
	{
		this.name = rsmd.getColumnName( nCol );
		this.type = rsmd.getColumnTypeName( nCol );
		this.precision = rsmd.getPrecision( nCol );
		this.scale = rsmd.getScale( nCol );
		this.key = false;
		this.nullable = ( rsmd.isNullable( nCol ) == ResultSetMetaData.columnNullable );
	}

	public ColumnInfo( ResultSet rset )
		throws SQLException
	{
		String var;
		this.name = rset.getString( "COLUMN_NAME" );
		this.type = rset.getString( "TYPE_NAME" );
		this.precision = rset.getInt( "COLUMN_SIZE" );
		this.scale = rset.getInt( "DECIMAL_DIGITS" );
		this.key = false;
		this.nullable = ( rset.getInt( "NULLABLE" ) == 0 );
		this.defValue = rset.getString( "COLUMN_DEF" );
		this.remarks = rset.getString( "REMARKS" );
	}

	public ColumnInfo( DbColumn col ) {
		String var;
		this.name = col.getName();
		this.type = col.getTypeName();
		this.javaType = col.getType();
		this.precision = col.getPrecision();
		this.scale = col.getScale();
		this.key = false;
		this.nullable = ( col.isNullable() != 0 );
	}

	public String getMethodName() {
		return getName().substring( 0, 1 ).toUpperCase()
			+ getName().substring( 1 ).toLowerCase();
	}

	static class TypeInfo {
		String	dbType;
		String	sqlType;
		String	fetchType;
		boolean	needPrecision;
		boolean	needScale;
		TypeInfo( String dbType, String sqlType, String fetchType,
				boolean needPrecision, boolean needScale )
		{
			this.dbType = dbType;
			this.sqlType = sqlType;
			this.fetchType = fetchType;
			this.needPrecision = needPrecision;
			this.needScale = needScale;
		}
	}

	static HashMap<String,TypeInfo> mapType = new HashMap<String,TypeInfo>();
	static {
mapType.put( "varchar2",
	new TypeInfo( "varchar2", "VARCHAR", "String", true, false ));
mapType.put( "bpchar",
	new TypeInfo( "bpchar", "CHAR", "String", true, false ));
mapType.put( "short",
	new TypeInfo( "short", "SMALLINT", "Short", false, false ));
mapType.put( "int2",
	new TypeInfo( "int2", "INTEGER", "Integer", false, false ));
mapType.put( "int4",
	new TypeInfo( "int4", "INTEGER", "Integer", false, false ));
mapType.put( "int8",
	new TypeInfo( "int8", "INTEGER", "Integer", false, false ));
mapType.put( "float4",
	new TypeInfo( "float4", "DOUBLE", "Double", false, false ));
mapType.put( "float8",
	new TypeInfo( "float8", "DOUBLE", "Double", false, false ));
mapType.put( "bool",
	new TypeInfo( "bool", "BOOLEAN", "Boolean", true, true ));
		mapType.put( "char",
			new TypeInfo( "char", "CHAR", "String", true, false ));
		mapType.put( "varchar",
			new TypeInfo( "varchar", "VARCHAR", "String", true, false ));
		mapType.put( "text",
			new TypeInfo( "text", "LONGVARCHAR", "String", true, false ));
		mapType.put( "integer",
			new TypeInfo( "integer", "INTEGER", "Integer", false, false ));
		mapType.put( "smallint",
			new TypeInfo( "smallint", "SMALLINT", "Short", false, false ));
		mapType.put( "float",
			new TypeInfo( "float", "DOUBLE", "Double", false, false ));
		mapType.put( "double",
			new TypeInfo( "double", "DOUBLE", "Double", false, false ));
		mapType.put( "decimal",
			new TypeInfo( "decimal", "DECIMAL", "BigDecimal", true, true ));
		mapType.put( "numeric",
			new TypeInfo( "numeric", "DECIMAL", "BigDecimal", true, true ));
		mapType.put( "number",
			new TypeInfo( "number", "DECIMAL", "BigDecimal", true, true ));
		mapType.put( "boolean",
			new TypeInfo( "boolean", "BOOLEAN", "Boolean", true, true ));

		mapType.put( "timestamp",
			new TypeInfo( "timestamp", "TIMESTAMP", "Timestamp", false, false ));
		mapType.put( "datetime",
			new TypeInfo( "datetime", "TIMESTAMP", "Timestamp", false, false ));
		mapType.put( "date",
			new TypeInfo( "date", "TIMESTAMP", "Timestamp", false, false ));
	}

	public String getSqlType() {
		return getSqlType( getType() );
	}
	public static String getSqlType( String type ) {
		String ret = null;
		TypeInfo info = (TypeInfo )mapType.get( type.toLowerCase());
		if ( info != null ) {
			ret = "Types." + info.sqlType;
		}
		return ret;
	}

	public String getFetchType() {
		return getFetchType( getType() );
	}
	public static String getFetchType( String type ) {
		String ret = null;
		TypeInfo info = (TypeInfo )mapType.get( type.toLowerCase());
		if ( info != null ) {
			ret = info.fetchType;
		}
		return ret;
	}

	public boolean needPrecision() {
		return needPrecision( getType() );
	}
	public static boolean needPrecision( String type ) {
		if ( type == null ) {
			return false;
		}
		boolean ret = false;
		TypeInfo info = (TypeInfo )mapType.get( type.toLowerCase());
		if ( info != null ) {
			ret = info.needPrecision;
		}
		return ret;
	}

	public boolean needScale() {
		return needScale( getType() );
	}
	public static boolean needScale( String type ) {
		if ( type == null ) {
			return false;
		}
		boolean ret = false;
		TypeInfo info = (TypeInfo )mapType.get( type.toLowerCase());
		if ( info != null ) {
			ret = info.needScale;
		}
		return ret;
	}

	String	name;	// カラム名
	String	type;	// DBの型
	int	javaType;	// java.sql.Typesの型
	int	size;	// カラムサイズ
	int	precision;	// 制度
	int	scale;	// 小数点
	boolean	key;	// キー
	boolean	nullable;	// Null可能
	String	defValue;	// デフォルト値
	String	remarks;	// コメント

	/**
	* カラム名を設定します。
	* @param	var		設定するデータ
	*/
	public void setName( String var ) {
		this.name = var;
	}
	/**
	* カラム名を取得します。
	* @return	カラム名
	*/
	public String getName() {
		return name;
	}

	/**
	* DBの型を設定します。
	* @param	var		設定するデータ
	*/
	public void setType( String var ) {
		this.type = var;
	}
	/**
	* DBの型を取得します。
	* @return	DBの型
	*/
	public String getType() {
		return type;
	}

	/**
	* java.sql.Typesの型を設定します。
	* @param	var		設定するデータ
	*/
	public void setJavaType( int var ) {
		this.javaType = var;
	}
	/**
	* java.sql.Typesの型を取得します。
	* @return	java.sql.Typesの型
	*/
	public int getJavaType() {
		return javaType;
	}

	/**
	* カラムサイズを設定します。
	* @param	var		設定するデータ
	*/
	public void setSize( int var ) {
		this.size = var;
	}
	/**
	* カラムサイズを取得します。
	* @return	カラムサイズ
	*/
	public int getSize() {
		return size;
	}

	/**
	* 制度を設定します。
	* @param	var		設定するデータ
	*/
	public void setPrecision( int var ) {
		this.precision = var;
	}
	/**
	* 制度を取得します。
	* @return	制度
	*/
	public int getPrecision() {
		return precision;
	}

	/**
	* 小数点を設定します。
	* @param	var		設定するデータ
	*/
	public void setScale( int var ) {
		this.scale = var;
	}
	/**
	* 小数点を取得します。
	* @return	小数点
	*/
	public int getScale() {
		return scale;
	}

	/**
	* キーを設定します。
	* @param	var		設定するデータ
	*/
	public void setKey( boolean var ) {
		this.key = var;
	}
	/**
	* キーを取得します。
	* @return	キー
	*/
	public boolean isKey() {
		return key;
	}

	/**
	* Null可能を設定します。
	* @param	var		設定するデータ
	*/
	public void setNullable( boolean var ) {
		this.nullable = var;
	}
	/**
	* Null可能を取得します。
	* @return	Null可能
	*/
	public boolean isNullable() {
		return nullable;
	}

	/**
	* デフォルト値を設定します。
	* @param	var		設定するデータ
	*/
	public void setDefValue( String var ) {
		this.defValue = var;
	}
	/**
	* デフォルト値を取得します。
	* @return	デフォルト値
	*/
	public String getDefValue() {
		return defValue;
	}

	/**
	* コメントを設定します。
	* @param	var		設定するデータ
	*/
	public void setRemarks( String var ) {
		this.remarks = var;
	}
	/**
	* コメントを取得します。
	* @return	コメント
	*/
	public String getRemarks() {
		return remarks;
	}

}
