package	jp.co.sds_corp.jdbrowser;

import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import jp.co.sds_corp.jdbrowser.connect.ConnectInfo;
import jp.co.sds_corp.jdbrowser.event.TableEvent;
import jp.co.sds_corp.jdbrowser.event.TableEventListener;
import jp.co.sds_corp.jdbrowser.procedure.ProcedureDesign;
import jp.co.sds_corp.jdbrowser.status.StatusPanel;
import jp.co.sds_corp.jdbrowser.table.QueryViewer;
import jp.co.sds_corp.jdbrowser.table.TableDesign;
import jp.co.sds_corp.jdbrowser.table.TableViewer;
import jp.co.sds_corp.jdbrowser.table.ViewDesign;
import jp.co.sds_corp.jdbrowser.table.ViewViewer;
import jp.co.sds_corp.util.db.DbAccess;
import jp.co.sds_corp.util.log.CmnLogger;

public class FrameManager
{
	static CmnLogger logger = CmnLogger.getInstance();
	static int openFrame = 0;
	final int maxFrame = 20;
	final int xOffset = 20, yOffset = 20;
	JFrame		parent;
	SetupParam	param;
	DbAccess	db;
	ConnectInfo	con;
	Vector<JInternalFrame>	vecPanel = new Vector<JInternalFrame>();
	HashMap<String,JInternalFrame>	mapPanel = new HashMap<String,JInternalFrame>();
	static Vector<JInternalFrame>	vecAll = new Vector<JInternalFrame>();
	Vector<TableEventListener>	vecLsn = new Vector<TableEventListener>();

	public FrameManager( JFrame parent, SetupParam param,
		DbAccess db, ConnectInfo con )
	{
		this.parent = parent;
		this.param = param;
		this.db = db;
		this.con = con;
	}

	public JInternalFrame getPanel( String name ) {
		JInternalFrame ret = (JInternalFrame )mapPanel.get( name );
		return ret;
	}

	public void addPanel( String name, JInternalFrame panel ) {
		mapPanel.put( name, panel );
	}

	public void removePanel( JInternalFrame panel ) {
		vecAll.remove( panel );
	}

	public void showPanel( JInternalFrame panel ) {
		if ( panel == null ) {
			if ( param.isDebug()) {
				logger.debug( this, "panel is null" );
			}
			return;
		}
		if ( vecPanel.indexOf( panel ) < 0 ) {
			panel.addInternalFrameListener( new InternalFrameAdapter() {
				public void internalFrameClosed( InternalFrameEvent ev ) {
					closeFrame( (JInternalFrame )ev.getSource() );
				}
			} );
			vecPanel.add( panel );
			locatePanel( panel );
		} else {
			try {
				panel.setVisible( true );
				panel.setIcon( false );
				panel.setSelected( true );
			} catch ( Exception ex ) {
			}
		}
	}

	public void locatePanel( JInternalFrame panel ) {
		parent.getContentPane().add( panel );
		panel.setLocation( xOffset*openFrame, yOffset*openFrame );
		openFrame++;
		if ( openFrame > maxFrame ) {
			openFrame = 0;
		}
		panel.setVisible( true );
		try {
			panel.setSelected( true );
		} catch ( Exception ex ) {
		}
		vecAll.add( panel );
	}

	public void closeFrame( NodeInfo.Type type, String name ) {
		switch ( type ) {
		case TABLE:
		case VIEW:
			closeFrame( "TABLEV:" + con.getDisplayName() + "." + name );
			closeFrame( "TABLED:" + con.getDisplayName() + "." + name );
			break;
		case PROCEDURE:
			closeFrame( "PROCV:" + con.getDisplayName() + "." + name );
			closeFrame( "PROCD:" + con.getDisplayName() + "." + name );
			break;
		}
	}

	public void closeFrame( String name ) {
		JInternalFrame panel = (JInternalFrame )mapPanel.get( name );
		if ( panel != null ) {
			panel.dispose();
			closeFrame( panel );
		}
	}

	public void closeFrame( JInternalFrame panel ) {
		vecAll.remove( panel );
		vecPanel.remove( panel );
		mapPanel.values().remove( panel );
	}

	public void closeAll() {
		for ( int i = vecPanel.size() - 1; i >= 0; i-- ) {
			JInternalFrame panel = (JInternalFrame )vecPanel.get( i );
			vecAll.remove( panel );
			panel.dispose();
		}
	}

	public static List<JInternalFrame> listFrame() {
		return vecAll;
	}

	public JInternalFrame showTableViewer( String catalog, String schema, String name )
		throws Exception
	{
		String key = "TABLEV:" + con.getDisplayName() + "." + name;
		TableViewer panel = (TableViewer )getPanel( key );
		if ( panel == null ) {
			panel = new TableViewer( parent, param, this );
			panel.select( db, con, catalog, schema, name );
			addPanel( key, panel );
		}
		showPanel( panel );
		return panel;
	}

	public JInternalFrame showTableDesigner( String catalog, String schema, String name )
		throws Exception
	{
		String key = "TABLED:" + con.getDisplayName() + "." + name;
		TableDesign panel = (TableDesign )getPanel( key );
		if ( panel == null ) {
			panel = new TableDesign( parent, param );
			panel.addTableEventListener( new TableEventListener() {
				public void tableChanged( TableEvent ev ) {
					fireTableChanged( ev );
				}
			} );
			panel.select( db, con, catalog, schema, name );
			addPanel( key, panel );
		}
		showPanel( panel );
		return panel;
	}

	public JInternalFrame showViewViewer( String catalog, String schema, String name )
		throws Exception
	{
		String key = "VIEWV:" + con.getDisplayName() + "." + name;
		ViewViewer panel = (ViewViewer )getPanel( key );
		if ( panel == null ) {
			panel = new ViewViewer( parent, param, this );
			panel.select( db, con, catalog, schema, name );
			addPanel( key, panel );
		}
		showPanel( panel );
		return panel;
	}

	public JInternalFrame showViewDesigner( String catalog, String schema, String name )
		throws Exception
	{
		String key = "VIEWD:" + con.getDisplayName() + "." + name;
		ViewDesign panel = (ViewDesign )getPanel( key );
		if ( panel == null ) {
			panel = new ViewDesign( parent, param );
			panel.select( db, con, catalog, schema, name );
			addPanel( key, panel );
		}
		showPanel( panel );
		return panel;
	}

	public JInternalFrame newTableDesigner( String catalog, String schema )
		throws Exception
	{
		TableDesign panel = new TableDesign( parent, param );
		panel.addTableEventListener( new TableEventListener() {
			public void tableChanged( TableEvent ev ) {
				fireTableChanged( ev );
			}
		} );
		panel.newTable( db, con, catalog, schema );
		showPanel( panel );
		return panel;
	}

	public JInternalFrame showProcedureDesigner( String catalog, String schema, String name )
		throws Exception
	{
		String key = "PROCV:" + con.getDisplayName() + "." + name;
		ProcedureDesign panel = (ProcedureDesign )getPanel( key );
		if ( panel == null ) {
			panel = new ProcedureDesign( parent, param );
			panel.select( db, con, catalog, schema, name );
			addPanel( key, panel );
		}
		showPanel( panel );
		return panel;
	}

	public JInternalFrame showQuery()
		throws Exception
	{
		String key = "QUERY:" + con.getDisplayName();
		QueryViewer panel = (QueryViewer )getPanel( key );
		if ( panel == null ) {
			panel = new QueryViewer( parent, param, db, con );
			addPanel( key, panel );
		}
		showPanel( panel );
		return panel;
	}

	public JInternalFrame showStatus()
		throws Exception
	{
		String key = "STATUS:" + con.getDisplayName();
		StatusPanel panel = (StatusPanel )getPanel( key );
		if ( panel == null ) {
			panel = new StatusPanel( parent, param );
			panel.setDb( db, con );
			addPanel( key, panel );
		}
		showPanel( panel );
		return panel;
	}

	public void addTableEventListener( TableEventListener lsn ) {
		vecLsn.add( lsn );
	}
	public void removeTableEventListener( TableEventListener lsn ) {
		vecLsn.remove( lsn );
	}
	public void fireTableChanged( TableEvent ev ) {
		for ( int i = 0; i < vecLsn.size(); i++ ) {
			TableEventListener lsn = (TableEventListener )vecLsn.get( i );
			lsn.tableChanged( ev );
		}
	}

}
