package	jp.co.sds_corp.jdbrowser;

import java.awt.Component;
import java.awt.Container;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JDesktopPane;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JToolBar;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.WindowConstants;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;
import javax.swing.plaf.FontUIResource;

import jp.co.sds_corp.jdbrowser.browser.BrowserPanel;
import jp.co.sds_corp.jdbrowser.setup.SetupDialog;
import jp.co.sds_corp.util.common.CmnProperty;
import jp.co.sds_corp.util.config.DOMConfiguration;
import jp.co.sds_corp.util.config.DOMWriter;
import jp.co.sds_corp.util.config.IConfigurable;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogger;
import jp.co.sds_corp.util.window.ExtFileFilter;
import jp.co.sds_corp.util.window.WindowUtility;

/**
* JDBrowserのメインクラスです。
*/
public class JDBrowser extends JFrame implements IConfigurable
{
	private static final long serialVersionUID = 1L;
	static CmnLogger logger = CmnLogger.getInstance();
	String			configFile;
	SetupParam		param;
	JDesktopPane	desktop;
	JMenuBar		menuBar;
	JMenu			winMenu;
	CmnProperty		prop;
	SetupDialog		setup;
	AboutDialog		about;
	Vector<BrowserPanel>	vecBrowser = new Vector<BrowserPanel>();

	public JDBrowser() {
		super();
		prop = new CmnProperty( "resource.jdbrowser" );
		setTitle( "JDBrowser" );
		setMenu();
		desktop = new JDesktopPane();
		setContentPane( desktop );

		Runtime.getRuntime().addShutdownHook( new Thread() {
			public void run() {
				onShutdown();
			}
		});
		addWindowListener( new WindowAdapter() {
			public void windowClosing( WindowEvent ev ) {
				String msg = prop.getString( "window.close.message" );
				String title = prop.getString( "window.close.title" );
				int nRet = JOptionPane.showConfirmDialog( JDBrowser.this,
							msg, title,
							JOptionPane.YES_NO_OPTION );
				if ( nRet == JOptionPane.YES_OPTION ) {
					System.exit( 0 );
				}
			}
		});
		setDefaultCloseOperation( WindowConstants.DO_NOTHING_ON_CLOSE );
	}

	public void onShutdown() {
		for ( int i = 0; i < vecBrowser.size(); i++ ) {
			BrowserPanel panel = (BrowserPanel )vecBrowser.get( i );
			panel.stop();
		}
	}

	public void init( IConfiguration config ) throws Exception {
		param = new SetupParam();
		param.init( config );
		setBounds( param.getBounds() );

		setup = new SetupDialog( this, param );
		setup.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				updateFont( param.getFont() );
				setBounds( param.getBounds());
			}
		} );
		about = new AboutDialog( this, param );
		updateFont( param.getFont() );
	}

	private void updateFont( Font font ) {
		FontUIResource fontui = new FontUIResource( font );
		UIDefaults defaultTable = UIManager.getLookAndFeelDefaults();
		Iterator itr = defaultTable.keySet().iterator();
		while ( itr.hasNext()) {
			Object obj = itr.next();
			if ( obj.toString().toLowerCase().endsWith( "font" )) {
				if ( obj.toString().startsWith( "Menu" )) {
					continue;
				}
				UIManager.put( obj, fontui );
			}
		}
		recursiveUpdateUI( this, true );
		recursiveUpdateUI( setup, true );
	}

	private void recursiveUpdateUI( Container p, boolean var ) {
		Component[] arr = p.getComponents();
		for ( int i = 0; i < arr.length; i++ ) {
			Component c = arr[i];
			if( c instanceof JComponent ) {
				JComponent jc = (JComponent )c;
				// JToolBar配下のJButtonは更新しない
				if ( jc instanceof JToolBar ) {
					var = false;
				}
				if ( var || !( jc instanceof JButton ) ) {
					jc.updateUI();
				}
				if ( jc.getComponentCount() > 0 ) {
					recursiveUpdateUI( jc, var );
				}
			}
		}
	}

	private void setMenu() {
		JMenu		menu, submenu;
		JMenuItem	menuItem;
		JCheckBoxMenuItem		cbMenuItem;
		JRadioButtonMenuItem	rbMenuItem;
		ButtonGroup group;

		String text;
		menuBar = new JMenuBar();

		// ファイル
		text = prop.getString( "menu1.text", null );
		if ( text != null ) {
			menu = new JMenu( text );
			menu.setMnemonic( KeyEvent.VK_F );
			menuBar.add( menu );

			text = prop.getString( "menu1.item1.text", null );
			if ( text != null ) {
				menuItem = new JMenuItem( text, KeyEvent.VK_C );
				menuItem.addActionListener( new ActionListener() {
					public void actionPerformed( ActionEvent ev ) {
						startBrowser();
					}
				});
				menu.add( menuItem );
				menu.addSeparator();
			}
			text = prop.getString( "menu1.item2.text", null );
			if ( text != null ) {
				menuItem = new JMenuItem( text, KeyEvent.VK_S );
				menuItem.addActionListener( new ActionListener() {
					public void actionPerformed( ActionEvent ev ) {
						setup();
					}
				});
				menu.add( menuItem );
			}
			text = prop.getString( "menu1.item3.text", null );
			if ( text != null ) {
				menuItem = new JMenuItem( text, KeyEvent.VK_A );
				menuItem.addActionListener( new ActionListener() {
					public void actionPerformed( ActionEvent ev ) {
						saveSetup();
					}
				});
				menu.add( menuItem );
			}
			menu.addSeparator();
			text = prop.getString( "menu1.item4.text", null );
			if ( text != null ) {
				menuItem = new JMenuItem( text, KeyEvent.VK_X );
				menuItem.addActionListener( new ActionListener() {
					public void actionPerformed( ActionEvent ev ) {
						closeFrame();
					}
				});
				menu.add( menuItem );
			}
		}

		// ウィンドウ
		text = prop.getString( "menu4.text", null );
		if ( text != null ) {
			menu = new JMenu( text );
			menu.setMnemonic( KeyEvent.VK_W );
			winMenu = menu;
			menu.addMenuListener( new MenuListener() {
				public void menuCanceled( MenuEvent ev ) {
				}
				public void menuDeselected( MenuEvent ev ) {
				}
				public void menuSelected( MenuEvent ev ) {
					selectWindowMenu();
				}
			});
			selectWindowMenu();
			menuBar.add( menu );
		}

		// ヘルプ
		text = prop.getString( "menu5.text", null );
		if ( text != null ) {
			menu = new JMenu( text );
			menu.setMnemonic( KeyEvent.VK_H );
			menuBar.add( menu );
			text = prop.getString( "menu5.item1.text", null );
			if ( text != null ) {
				menuItem = new JMenuItem( text, KeyEvent.VK_A );
				menuItem.addActionListener( new ActionListener() {
					public void actionPerformed( ActionEvent ev ) {
						showAbout();
					}
				});
				menu.add( menuItem );
			}
		}

		setJMenuBar( menuBar );
	}

	public void startBrowser() {
		BrowserPanel panel = new BrowserPanel( this, param );
		boolean bRet = panel.start();
		if ( bRet ) {
			vecBrowser.add( panel );
		}
	}

	void setup() {
		WindowUtility.center( this, setup );
		param.setBounds( getBounds() );
		setup.setVisible( true );
	}

	void closeFrame() {
		processWindowEvent( new WindowEvent( this, WindowEvent.WINDOW_CLOSING ));
	}

	void saveSetup() {
		String file = getSaveFile();
		if ( file == null ) {
			return;
		}
		try {
			DOMConfiguration config = DOMConfiguration.createDocument( "JDBrowser" );
			param.saveConfig( config );
			Writer out = new OutputStreamWriter(
				new FileOutputStream( file ), "UTF-8" );
			DOMWriter writer = new DOMWriter( out );
			writer.write( config, "JDBrowser", true );
			// writer.write( config, "JDBrowser", null );
			out.flush();
			out.close();
		} catch ( Exception ex ) {
ex.printStackTrace();
		}
	}

	private String getSaveFile() {
		String[] ext = { "xml" };
		String text = prop.getString( "setup.save.filter" );
		JFileChooser fc = new JFileChooser( configFile ) {
			public void approveSelection() {
				File sel = getSelectedFile();
				if ( getFileFilter() instanceof ExtFileFilter ) {
					if ( !sel.getPath().toLowerCase().endsWith( ".xml" )) {
						sel = new File( sel.getPath() + ".xml" );
						setSelectedFile( sel );
					}
				}
				if ( sel.exists() ) {
					String title = prop.getString( "setup.save.exists.title" );
					String msg = prop.getString( "setup.save.exists.message" );
					int nRet = JOptionPane.showConfirmDialog( this,
						msg, title,
						JOptionPane.YES_NO_OPTION );
					if ( nRet != JOptionPane.YES_OPTION ) {
						return;
					}
				}
				super.approveSelection();
			}
		};
		if ( configFile != null ) {
			fc.setSelectedFile( new File( configFile ));
		}
		fc.setFileFilter( new ExtFileFilter( text, ext ) );
		int nRet = fc.showSaveDialog( this );
		if ( nRet != JFileChooser.APPROVE_OPTION ) {
			return null;
		}
		return fc.getSelectedFile().toString();
	}

	public void selectWindowMenu() {
		JMenuItem menuItem = null;
		String text;
		winMenu.removeAll();
		text = prop.getString( "menu4.item1.text", null );
		if ( text != null ) {
			menuItem = new JMenuItem( text, KeyEvent.VK_I );
			menuItem.addActionListener( new ActionListener() {
				public void actionPerformed( ActionEvent ev ) {
					iconFrame();
				}
			} );
			winMenu.add( menuItem );
		}
		java.util.List<JInternalFrame> list = FrameManager.listFrame();
		if ( list.size() == 0 ) {
			if ( menuItem != null ) {
				menuItem.setEnabled( false );
			}
			return;
		}
		winMenu.addSeparator();
		for ( int i = 0; i < list.size(); i++ ) {
			JInternalFrame panel = (JInternalFrame )list.get( i );
			menuItem = new JMenuItem( panel.getTitle() );
			menuItem.addActionListener( new FrameSelector( panel ));
			winMenu.add( menuItem );
		}
	}

	class FrameSelector implements ActionListener {
		JInternalFrame frame;
		FrameSelector( JInternalFrame frame ) {
			this.frame = frame;
		}
		public void actionPerformed( ActionEvent ev ) {
			try {
				frame.setVisible( true );
				if ( frame.isIcon()) {
					frame.setIcon( false );
				}
				frame.setSelected( true );
			} catch ( Exception ex ) {
			}
		}
	}

	protected void iconFrame() {
		java.util.List<JInternalFrame> list = FrameManager.listFrame();
		if ( list.size() == 0 ) {
			return;
		}
		for ( int i = 0; i < list.size(); i++ ) {
			JInternalFrame panel = (JInternalFrame )list.get( i );
			if ( panel.isSelected()) {
				try {
					panel.setIcon( true );
				} catch ( Exception ex ) {
				}
				break;
			}
		}
	}

	public void showAbout() {
		about.setVisible( true );
	}
	public static void main( String argv[] ) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception exc) {
			System.err.println( "Error loading L&F: " + exc);
		}
		try {
			String file = null;
			try {
				file = System.getProperty( "JDBrowser.config" );
			} catch ( Exception fex ) {
			}
			if ( file == null ) {
				file = "jdbrowser.xml";
			}
			DOMConfiguration config = new DOMConfiguration( file );

			JDBrowser win = new JDBrowser();
			win.configFile = file;
			win.init( config );
			win.setVisible( true );
			win.startBrowser();
			win.toFront();
		} catch ( Throwable ex ) {
			logger.error( ex, "initialize error", ex );
			JOptionPane.showMessageDialog( null,
				"Error occurred.\n" + ex,
				"Error",
				JOptionPane.ERROR_MESSAGE );
			System.exit( 9 );
		}
	}
}
