package	jp.co.sds_corp.jdbrowser;

import jp.co.sds_corp.jdbrowser.connect.ConnectInfo;

/**
* {@link jp.co.sds_corp.jdbrowser.browser.BrowserTree}のNode情報を保持します。
*/
public class NodeInfo
{
	public enum Level {
		ROOT,
		TYPE,
		LEAF
	};

	public enum Type {
		TABLE,
		INDEX,
		PKEY,
		PROCEDURE,
		VIEW
	};

	// for root node
	public NodeInfo( String name ) {
		this.level = Level.ROOT;
		this.name = name;
	}

	// for type node
	public NodeInfo( Type type, String name ) {
		this.level = Level.TYPE;
		this.type = type;
		this.name = name;
	}

	// for leaf
	public NodeInfo( Type type, ConnectInfo con,
					String catalog, String schema, String name )
	{
		this.level = Level.LEAF;
		this.type = type;
		this.con = con;
		this.catalog = catalog;
		this.schema= schema;
		this.name = name;
	}

	public NodeInfo( Level level, Type type, ConnectInfo con,
					String catalog, String schema, String name )
	{
		this.level = level;
		this.type = type;
		this.con = con;
		this.catalog = catalog;
		this.schema= schema;
		this.name = name;
	}

	public String toString() {
		return name;
	}

	Level	level;	// 表示レベル
	Type	type;	// オブジェクト種別
	ConnectInfo	con;	// DBの接続先
	String	catalog;	// DBのCatalog
	String	schema;	// DBのSchema
	String	name;	// 名称

	/**
	* 表示レベルを設定します。
	* @param	var		設定するデータ
	*/
	public void setLevel( Level var ) {
		this.level = var;
	}
	/**
	* 表示レベルを取得します。
	* @return	表示レベル
	*/
	public Level getLevel() {
		return level;
	}

	/**
	* オブジェクト種別を設定します。
	* @param	var		設定するデータ
	*/
	public void setType( Type var ) {
		this.type = var;
	}
	/**
	* オブジェクト種別を取得します。
	* @return	オブジェクト種別
	*/
	public Type getType() {
		return type;
	}

	/**
	* DBの接続先を設定します。
	* @param	var		設定するデータ
	*/
	public void setCon( ConnectInfo var ) {
		this.con = var;
	}
	/**
	* DBの接続先を取得します。
	* @return	DBの接続先
	*/
	public ConnectInfo getCon() {
		return con;
	}

	/**
	* DBのCatalogを設定します。
	* @param	var		設定するデータ
	*/
	public void setCatalog( String var ) {
		this.catalog = var;
	}
	/**
	* DBのCatalogを取得します。
	* @return	DBのCatalog
	*/
	public String getCatalog() {
		return catalog;
	}

	/**
	* DBのSchemaを設定します。
	* @param	var		設定するデータ
	*/
	public void setSchema( String var ) {
		this.schema = var;
	}
	/**
	* DBのSchemaを取得します。
	* @return	DBのSchema
	*/
	public String getSchema() {
		return schema;
	}

	/**
	* 名称を設定します。
	* @param	var		設定するデータ
	*/
	public void setName( String var ) {
		this.name = var;
	}
	/**
	* 名称を取得します。
	* @return	名称
	*/
	public String getName() {
		return name;
	}

}
