package	jp.co.sds_corp.jdbrowser;

import java.awt.Font;
import java.awt.Rectangle;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import jp.co.sds_corp.jdbrowser.connect.ConnectInfo;
import jp.co.sds_corp.jdbrowser.generator.CsvLoader;
import jp.co.sds_corp.jdbrowser.generator.CsvParam;
import jp.co.sds_corp.jdbrowser.generator.CsvSaver;
import jp.co.sds_corp.jdbrowser.generator.DocParam;
import jp.co.sds_corp.jdbrowser.generator.DocumentGenerator;
import jp.co.sds_corp.jdbrowser.generator.GeneratorFactory;
import jp.co.sds_corp.jdbrowser.generator.JavaGenerator;
import jp.co.sds_corp.jdbrowser.generator.JavaParam;
import jp.co.sds_corp.util.config.IConfigurable;
import jp.co.sds_corp.util.config.IConfiguration;

/**
* JDBrowserの設定を保持するクラスです。
<pre>
&lt;Setup  
	Height="800" Width="1000" X="30" Y="10"
	debug="true"
	rowset="com.sun.rowset.CachedRowSetImpl"&gt;
	&lt;Font name="Dialog" size="14" style="PLAIN"/&gt;
&lt;/Setup&gt;

&lt;Java&gt;要素については、{@link GeneratorFactory}、{@link JavaGenerator}を参照してください。
&lt;CSV&gt;要素については、{@link CsvLoader}、{@link CsvSaver}を参照してください。
&lt;Document&gt;要素については、{@link GeneratorFactory}、{@link DocumentGenerator}を参照してください。
&lt;Connection&gt;要素については、{@link ConnectInfo}を参照してください。
</pre>
*/
public class SetupParam implements IConfigurable
{
	IConfiguration config;
	int		nWidth = 800;
	int		nHeight = 400;
	int		nX = 0;
	int		nY = 0;
	Font	font;
	String	rowSet;
	boolean	debug = false;
	Map<String,ConnectInfo>		mapCon = new HashMap<String,ConnectInfo>();
	Vector<ConnectInfo>	vecCon = new Vector<ConnectInfo>();
	boolean	includeSystem;	// includeSystem
	JavaParam	javaParam;
	DocParam	docParam;
	CsvParam	csvParam;

	public void init( IConfiguration config ) throws Exception {
		this.config = config;
		IConfiguration c;
		initSetup( config.getChild( "Setup" ) );
		initJava( config.getChild( "Java" ) );
		initCsv( config.getChild( "CSV" ) );
		initDoc( config.getChild( "Document" ) );

		c = config.getChild( "Connection" );
		initConnection( c );
	}

	protected void initSetup( IConfiguration config ) throws Exception {
		if ( config == null ) {
			return;
		}
		String val;
		val = config.getAttributes().get( "X", null );
		if ( val != null ) {
			try {
				nX = Integer.valueOf( val ).intValue();
			} catch ( NumberFormatException ex ) {}
		}
		val = config.getAttributes().get( "Y", null );
		if ( val != null ) {
			try {
				nY = Integer.valueOf( val ).intValue();
			} catch ( NumberFormatException ex ) {}
		}
		val = config.getAttributes().get( "Width", null );
		if ( val != null ) {
			try {
				nWidth = Integer.valueOf( val ).intValue();
			} catch ( NumberFormatException ex ) {}
		}
		val = config.getAttributes().get( "Height", null );
		if ( val != null ) {
			try {
				nHeight = Integer.valueOf( val ).intValue();
			} catch ( NumberFormatException ex ) {}
		}
		val = config.getAttributes().get( "debug", "false" );
		if ( val != null && val.equals( "true" )) {
			debug = true;
		}
		rowSet = config.getAttributes().get( "rowset", null );

		IConfiguration c = config.getChild( "Font" );
		if ( c != null ) {
			initFont( c );
		}
	}

	protected void initFont( IConfiguration config ) throws Exception {
		IConfiguration attr = config.getAttributes();
		String name = attr.get( "name" );
		int nStyle;
		String style = attr.get( "style", "PLAIN" );
		if ( style.equals( "ITALIC" )) {
			nStyle = Font.ITALIC;
		} else if ( style.equals( "BOLD" )) {
			nStyle = Font.BOLD;
		} else if ( style.equals( "BOLD+ITALIC" )) {
			nStyle = Font.BOLD + Font.ITALIC;
		} else {
			nStyle = Font.PLAIN;
		}
		int nSize = attr.getInteger( "size", 14 );
		font = new Font( name, nStyle, nSize );
	}

	protected void initConnection( IConfiguration config ) throws Exception {
		Iterator<IConfiguration> itr = config.getChilds( "db" );
		while ( itr.hasNext()) {
			IConfiguration c = (IConfiguration )itr.next();
			ConnectInfo info = new ConnectInfo( c );
			if ( mapCon.get( info.getName() ) != null ) {
				throw( new Exception( "connection name duplicated." ));
			}
			mapCon.put( info.getName(), info );
			vecCon.add( info );
		}
	}

	protected void initJava( IConfiguration config ) throws Exception {
		javaParam = new JavaParam( config );
	}

	protected void initDoc( IConfiguration config ) throws Exception {
		docParam = new DocParam( config );
	}

	protected void initCsv( IConfiguration config ) throws Exception {
		csvParam = new CsvParam( config );
	}

	public JavaParam getJavaParam() {
		return javaParam;
	}

	public DocParam getDocParam() {
		return docParam;
	}

	public CsvParam getCsvParam() {
		return csvParam;
	}

	/**
	* 設定情報を保存します。
	* @param	config	設定情報
	*/
	public void saveConfig( IConfiguration config ) throws Exception {
		IConfiguration c;
		c = config.addChild( "Setup" );
		Rectangle rect = getBounds();
		c.setAttribute( "X", String.valueOf( nX ));
		c.setAttribute( "Y", String.valueOf( nY ));
		c.setAttribute( "Width", String.valueOf( nWidth ));
		c.setAttribute( "Height", String.valueOf( nHeight ));
		if ( rowSet != null && rowSet.length() > 0 ) {
			c.setAttribute( "rowset", rowSet );
		}
		c.setAttribute( "debug", String.valueOf( isDebug() ));
		if ( font != null ) {
			IConfiguration fc = c.addChild( "Font" );
			saveFont( fc );
		}

		if ( javaParam != null ) {
			c = config.addChild( "Java" );
			javaParam.saveConfig( c );
		}
		if ( csvParam != null ) {
			c = config.addChild( "CSV" );
			csvParam.saveConfig( c );
		}
		if ( docParam != null ) {
			c = config.addChild( "Document" );
			docParam.saveConfig( c );
		}
		c = config.addChild( "Connection" );
		saveConnection( c );
	}

	protected void saveFont( IConfiguration c ) throws Exception {
		c.setAttribute( "name", font.getName() );
		c.setAttribute( "size", String.valueOf( font.getSize()) );
		String s;
		if ( font.isItalic()) {
			if ( font.isBold()) {
				s = "BOLD+ITALIC";
			} else {
				s = "ITALIC";
			}
		} else {
			if ( font.isBold()) {
				s = "BOLD";
			} else {
				s = "PLAIN";
			}
		}
		c.setAttribute( "style", s );
	}

	protected void saveConnection( IConfiguration config ) throws Exception {
		IConfiguration c;
		for ( int i = 0; i < vecCon.size(); i++ ) {
			ConnectInfo info = (ConnectInfo )vecCon.get( i );
			c = config.addChild( "db" );
			info.saveConfig( c );
		}
	}

	public Rectangle getBounds() {
		Rectangle ret = new Rectangle( nX, nY, nWidth, nHeight );
		return ret;
	}
	public void setBounds( Rectangle rect ) {
		nX = (int )rect.getX();
		nY = (int )rect.getY();
		nWidth = (int )rect.getWidth();
		nHeight = (int )rect.getHeight();
	}

	public Vector<ConnectInfo> listConnection() {
		return vecCon;
	}
	public Map<String,ConnectInfo> mapConnection() {
		return mapCon;
	}

	/**
	* fontを設定します。
	* @param	var		設定するデータ
	*/
	public void setFont( Font var ) {
		this.font = var;
	}
	/**
	* fontを取得します。
	* @return	font
	*/
	public Font getFont() {
		return font;
	}

	/**
	* debugフラグを設定します。
	* @param	var		設定するデータ
	*/
	public void setDebug( boolean var ) {
		this.debug = var;
	}
	/**
	* debugフラグを取得します。
	* @return	debugフラグ
	*/
	public boolean isDebug() {
		return debug;
	}

	/**
	* includeSystemを設定します。
	* @param	var		設定するデータ
	*/
	public void setIncludeSystem( boolean var ) {
		this.includeSystem = var;
	}
	/**
	* includeSystemを取得します。
	* @return	includeSystem
	*/
	public boolean isIncludeSystem() {
		return includeSystem;
	}

	/**
	* rowsetクラスを設定します。
	* @param	var		設定するデータ
	*/
	public void setRowSet( String var ) {
		this.rowSet = var;
	}
	/**
	* rowsetクラスを取得します。
	* @return	rowsetクラス
	*/
	public String getRowSet() {
		return rowSet;
	}

}
