package	jp.co.sds_corp.jdbrowser.browser;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;
import javax.swing.tree.TreePath;

import jp.co.sds_corp.jdbrowser.AboutDialog;
import jp.co.sds_corp.jdbrowser.FrameManager;
import jp.co.sds_corp.jdbrowser.NodeInfo;
import jp.co.sds_corp.jdbrowser.SetupParam;
import jp.co.sds_corp.jdbrowser.connect.ConnectDialog;
import jp.co.sds_corp.jdbrowser.connect.ConnectInfo;
import jp.co.sds_corp.jdbrowser.event.TableEvent;
import jp.co.sds_corp.jdbrowser.event.TableEventListener;
import jp.co.sds_corp.jdbrowser.generator.CsvLoader;
import jp.co.sds_corp.jdbrowser.generator.CsvSaver;
import jp.co.sds_corp.jdbrowser.generator.GeneratorFactory;
import jp.co.sds_corp.jdbrowser.generator.IGenerator;
import jp.co.sds_corp.jdbrowser.util.BrowserUtil;
import jp.co.sds_corp.util.common.CmnProperty;
import jp.co.sds_corp.util.db.DbAccess;
import jp.co.sds_corp.util.log.CmnLogger;
import jp.co.sds_corp.util.window.WindowUtility;

/**
* DBの情報を表示します。
*/
public class BrowserPanel extends JInternalFrame
{
	private static final long serialVersionUID = 1L;
	static CmnLogger logger = CmnLogger.getInstance();
	JFrame		parent;
	SetupParam	param;
	CmnProperty	prop;
	AboutDialog	about;
	ConnectDialog	conDlg;
	JPopupMenu	defMenu;
	JPopupMenu	tblMenu;
	JPopupMenu	procMenu;
	JPopupMenu	nodeMenu;
	NodeInfo	selInfo;
	JComboBox	CMB_SCH = new JComboBox();
	JComboBox	CMB_CAT = new JComboBox();
	JToolBar	toolBar;
	JButton		BTN_REFRESH;
	JButton		BTN_ADD;
	JButton		BTN_DEL;
	JCheckBox	CHK_INCSYS;
	BrowserTree	tree;
	DbAccess	db = null;
	ConnectInfo	con;
	String	catalog;
	String	schema;
	Vector<String>	vecSch = new Vector<String>();
	Vector<String>	vecCat = new Vector<String>();
	String	errTitle;
	String	errMsg;
	FrameManager frmMng;

	public BrowserPanel( Window parent, SetupParam param ) {
		super( "", true, true, true, true );
		this.parent = (JFrame )parent;
		this.param = param;
		prop = new CmnProperty( "resource.jdbrowser" );
		errTitle = prop.getString( "browser.err.title" );
		errMsg = prop.getString( "browser.err.message" );
		about = new AboutDialog( parent, param );
		conDlg = new ConnectDialog( param );
		createControls();

		setDefaultCloseOperation( JInternalFrame.DO_NOTHING_ON_CLOSE );
		addInternalFrameListener( new InternalFrameAdapter() {
			public void internalFrameClosing( InternalFrameEvent ev ) {
				String msg = prop.getString( "browser.close.message" );
				String title = prop.getString( "browser.close.title" );
				int nRet = JOptionPane.showConfirmDialog( BrowserPanel.this,
							msg, title,
							JOptionPane.YES_NO_OPTION );
				if ( nRet == JOptionPane.YES_OPTION ) {
					BrowserPanel.this.dispose();
				}
			}
			public void internalFrameClosed( InternalFrameEvent ev ) {
				closed();
			}
		});
	}

	protected void createControls() {
		setSize( 300, 500 );

		createDefaultPopup();
		createTablePopup();
		createProcPopup();
		createNodePopup();

		Box panel1 = new Box( BoxLayout.Y_AXIS );
		Box panel2 = new Box( BoxLayout.X_AXIS );
		panel2.setMaximumSize( new Dimension( 1000, 10 ));
		panel2.add( new JLabel( "Catalog:" ));
		panel2.add( CMB_CAT );
		panel2.add( new JLabel( "Schema:" ));
		panel2.add( CMB_SCH );
		panel1.add( panel2 );

		JPanel panel3 = new JPanel( new BorderLayout() );
		toolBar = new JToolBar( "toolbar" );
		String text;
		BTN_REFRESH = new JButton();
		text = prop.getString( "browser.refresh.tooltip" );
		BTN_REFRESH.setToolTipText( text );
		WindowUtility.setButtonIcon( this, BTN_REFRESH,
			"/resource/refresh.png", "/resource/refresh_d.png" );
		BTN_REFRESH.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				refreshTree();
			}
		} );
		toolBar.add( BTN_REFRESH );
		toolBar.addSeparator();
		BTN_ADD = new JButton();
		WindowUtility.setButtonIcon( this, BTN_ADD, "/resource/create.png" );
		text = prop.getString( "browser.add.tooltip" );
		BTN_ADD.setToolTipText( text );
		BTN_ADD.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				createObject();
			}
		} );
		toolBar.add( BTN_ADD );
		BTN_DEL = new JButton();
		WindowUtility.setButtonIcon( this, BTN_DEL, "/resource/delete.png" );
		text = prop.getString( "browser.del.tooltip" );
		BTN_DEL.setToolTipText( text );
		BTN_DEL.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				deleteObject();
			}
		} );
		toolBar.add( BTN_DEL );
		text = prop.getString( "browser.incsys.text" );
		CHK_INCSYS = new JCheckBox( text );
		if ( param.isIncludeSystem()) {
			CHK_INCSYS.setSelected( true );
		}
		CHK_INCSYS.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				param.setIncludeSystem( !param.isIncludeSystem());
				refreshTree();
			}
		} );
		toolBar.add( CHK_INCSYS );
		panel3.add( toolBar, BorderLayout.NORTH );

		tree = new BrowserTree( param );
		tree.addMouseListener( new MouseAdapter() {
			public void mouseClicked( MouseEvent ev ) {
				if ( SwingUtilities.isRightMouseButton( ev )) {
					showPopup( ev );
					return;
				}
				if ( ev.getClickCount() == 2 ) {
					treeDoubleClicked( ev );
					return;
				}
			}
		} );
		JScrollPane sp = new JScrollPane( tree );
		panel3.add( sp, BorderLayout.CENTER );
		panel1.add( panel3 );
		getContentPane().add( panel1 );
	}

	protected void createDefaultPopup() {
		defMenu = new JPopupMenu();
		JMenuItem   item;
		String  text;
		text = prop.getString( "browser.default.menu1.text" );
		item = new JMenuItem( text, KeyEvent.VK_Q );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				showQuery();
			}
		});
		defMenu.add( item );
		text = prop.getString( "browser.default.menu2.text" );
		item = new JMenuItem( text, KeyEvent.VK_W );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				showStatus();
			}
		});
		defMenu.add( item );
	}

	protected void createNodePopup() {
		nodeMenu = new JPopupMenu();
		JMenuItem   item;
		String  text;
		text = prop.getString( "browser.node.menu1.text" );
		item = new JMenuItem( text, KeyEvent.VK_C );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				createObject();
			}
		});
		nodeMenu.add( item );
		text = prop.getString( "browser.node.menu2.text" );
		item = new JMenuItem( text, KeyEvent.VK_Q );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				showQuery();
			}
		});
		nodeMenu.add( item );
		text = prop.getString( "browser.node.menu3.text" );
		item = new JMenuItem( text, KeyEvent.VK_W );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				showStatus();
			}
		});
		nodeMenu.add( item );
	}

	protected void createTablePopup() {
		tblMenu = new JPopupMenu();
		JMenuItem   item;
		String  text;
		text = prop.getString( "browser.table.menu1.text" );
		item = new JMenuItem( text, KeyEvent.VK_E );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				showTableViewer();
			}
		});
		tblMenu.add( item );
		text = prop.getString( "browser.table.menu2.text" );
		item = new JMenuItem( text, KeyEvent.VK_D );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				showTableDesigner();
			}
		});
		tblMenu.add( item );
		text = prop.getString( "browser.table.menu3.text" );
		item = new JMenuItem( text, KeyEvent.VK_N );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				editName();
			}
		});
		tblMenu.add( item );
		text = prop.getString( "browser.table.menu4.text" );
		item = new JMenuItem( text, KeyEvent.VK_C );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				createObject();
			}
		});
		tblMenu.add( item );
		text = prop.getString( "browser.table.menu5.text" );
		item = new JMenuItem( text, KeyEvent.VK_R );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				deleteObject();
			}
		});
		tblMenu.add( item );
		tblMenu.addSeparator();
		text = prop.getString( "browser.table.menu6.text" );
		item = new JMenuItem( text, KeyEvent.VK_S );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				saveCsv();
			}
		});
		tblMenu.add( item );
		text = prop.getString( "browser.table.menu7.text" );
		item = new JMenuItem( text, KeyEvent.VK_L );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				loadCsv();
			}
		});
		tblMenu.add( item );
		tblMenu.addSeparator();
		text = prop.getString( "browser.table.menu8.text" );
		item = new JMenuItem( text, KeyEvent.VK_J );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				generateJava();
			}
		});
		tblMenu.add( item );
		text = prop.getString( "browser.table.menu9.text" );
		item = new JMenuItem( text, KeyEvent.VK_O );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				generateDocument();
			}
		});
		tblMenu.add( item );
		tblMenu.addSeparator();
		text = prop.getString( "browser.table.menu10.text" );
		item = new JMenuItem( text, KeyEvent.VK_Q );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				showQuery();
			}
		});
		tblMenu.add( item );
		text = prop.getString( "browser.table.menu11.text" );
		item = new JMenuItem( text, KeyEvent.VK_W );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				showStatus();
			}
		});
		tblMenu.add( item );
	}

	protected void createProcPopup() {
		procMenu = new JPopupMenu();
		JMenuItem   item;
		String  text;
		text = prop.getString( "browser.procedure.menu1.text" );
		item = new JMenuItem( text, KeyEvent.VK_D );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				showProcedureDesigner();
			}
		});
		procMenu.add( item );
		text = prop.getString( "browser.procedure.menu2.text" );
		item = new JMenuItem( text, KeyEvent.VK_R );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				deleteObject();
			}
		});
		procMenu.add( item );
		procMenu.addSeparator();
		text = prop.getString( "browser.procedure.menu3.text" );
		item = new JMenuItem( text, KeyEvent.VK_Q );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				showQuery();
			}
		});
		procMenu.add( item );
		text = prop.getString( "browser.procedure.menu4.text" );
		item = new JMenuItem( text, KeyEvent.VK_W );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				showStatus();
			}
		});
		procMenu.add( item );
	}

	private void error( Exception ex, boolean rollback ) {
		if ( rollback ) {
			try {
				db.rollback();
			} catch ( Exception ex2 ) {
			}
		}
ex.printStackTrace();
		JOptionPane.showMessageDialog( this,
			errMsg + "\n" + ex, errTitle,
			JOptionPane.ERROR_MESSAGE );
	}

	protected void refreshTree() {
		try {
			tree.select( db, con, catalog, schema );
		} catch ( Exception ex ) {
			error( ex, true );
		}
	}

	protected void editName() {
		TreePath path = tree.getSelectionPath();
		tree.startEditingAtPath( path );
	}

	protected void createObject() {
		try {
			TreePath path = tree.getSelectionPath();
			if ( path == null ) {
				return;
			}
			NodeInfo info = tree.getNodeInfo( path );
			if ( info.getType() != NodeInfo.Type.TABLE ) {
				return;
			}
			frmMng.newTableDesigner( info.getCatalog(), info.getSchema());
		} catch ( Exception ex ) {
			error( ex, true );
		}
	}

	protected void deleteObject() {
		try {
			JInternalFrame panel = null;
			TreePath[] paths = tree.getSelectionPaths();
			if ( paths == null ) {
				return;
			}
			for ( int i = 0; i < paths.length; i++ ) {
				NodeInfo info = tree.getNodeInfo( paths[i] );
				if ( info.getLevel() == NodeInfo.Level.LEAF &&
					( info.getType() == NodeInfo.Type.TABLE ||
						info.getType() == NodeInfo.Type.VIEW ))
				{
					deleteObject( info );
				}
			}
		} catch ( Exception ex ) {
			error( ex, true );
		}
	}
	protected void deleteObject( NodeInfo info ) throws Exception {
		String msg = prop.getString( "browser.del.message" );
		String title = prop.getString( "browser.del.title" );
		switch ( info.getType()) {
		case TABLE:
			msg += " TABLE [" + info.getName() + "]";
			break;
		case VIEW:
			msg += " VIEW [" + info.getName() + "]";
			break;
		case PROCEDURE:
			msg += " PROCEDURE [" + info.getName() + "]";
			break;
		default:
			return;
		}
		int nRet = JOptionPane.showConfirmDialog( BrowserPanel.this,
					msg, title,
					JOptionPane.YES_NO_OPTION );
		if ( nRet != JOptionPane.YES_OPTION ) {
			return;
		}
		String sql = "";
		switch ( info.getType()) {
		case TABLE:
			sql = "DROP TABLE " + info.getName();
			break;
		case VIEW:
			sql = "DROP VIEW " + info.getName();
			break;
		case PROCEDURE:
			sql = "DROP PROCEDURE " + info.getName();
			break;
		}
		db.execute( sql );
		db.commit();
		frmMng.closeFrame( info.getType(), info.getName() );
		tree.select( db, con, catalog, schema );
	}

	protected void showTableViewer( NodeInfo info ) throws Exception {
		if ( info.getLevel() != NodeInfo.Level.LEAF ) {
			return;
		}
		if ( info.getType() == NodeInfo.Type.TABLE ) {
			frmMng.showTableViewer( info.getCatalog(), info.getSchema(), info.getName() );
		} else
		if ( info.getType() == NodeInfo.Type.VIEW ) {
			frmMng.showViewViewer( info.getCatalog(), info.getSchema(), info.getName() );
		}
	}
	protected void showTableViewer() {
		TreePath[] paths = tree.getSelectionPaths();
		for ( int i = 0; i < paths.length; i++ ) {
			NodeInfo info = tree.getNodeInfo( paths[i] );
			try {
				showTableViewer( info );
			} catch ( Exception ex ) {
				error( ex, true );
				return;
			}
		}
	}

	protected void showTableDesigner( NodeInfo info ) throws Exception {
		if ( info.getLevel() != NodeInfo.Level.LEAF ) {
			return;
		}
		if ( info.getType() == NodeInfo.Type.TABLE ) {
			frmMng.showTableDesigner( info.getCatalog(), info.getSchema(), info.getName() );
		} else
		if ( info.getType() == NodeInfo.Type.VIEW ) {
			frmMng.showViewDesigner( info.getCatalog(), info.getSchema(), info.getName() );
		}
	}
	protected void showTableDesigner() {
		TreePath[] paths = tree.getSelectionPaths();
		for ( int i = 0; i < paths.length; i++ ) {
			NodeInfo info = tree.getNodeInfo( paths[i] );
			try {
				showTableDesigner( info );
			} catch ( Exception ex ) {
				error( ex, true );
				return;
			}
		}
	}

	protected void showProcedureDesigner( NodeInfo info ) throws Exception {
		if ( info.getLevel() != NodeInfo.Level.LEAF ) {
			return;
		}
		if ( info.getType() != NodeInfo.Type.PROCEDURE ) {
			return;
		}
		frmMng.showProcedureDesigner( info.getCatalog(), info.getSchema(), info.getName() );
	}
	protected void showProcedureDesigner() {
		TreePath[] paths = tree.getSelectionPaths();
		for ( int i = 0; i < paths.length; i++ ) {
			NodeInfo info = tree.getNodeInfo( paths[i] );
			try {
				showProcedureDesigner( info );
			} catch ( Exception ex ) {
				error( ex, true );
				return;
			}
		}
	}

	protected void showQuery() {
		try {
			frmMng.showQuery();
		} catch ( Exception ex ) {
			error( ex, true );
			return;
		}
	}

	protected void showStatus() {
		try {
			frmMng.showStatus();
		} catch ( Exception ex ) {
			error( ex, true );
			return;
		}
	}

	public void showAbout() {
		about.setVisible( true );
	}

	protected void saveCsv() {
		TreePath[] paths = tree.getSelectionPaths();
		JFileChooser fc = null;
		if ( paths.length > 1 ) {
			fc = new JFileChooser( "." );
			fc.setFileSelectionMode( JFileChooser.DIRECTORIES_ONLY );
			int nRet = fc.showSaveDialog( parent );
			if ( nRet != JFileChooser.APPROVE_OPTION ) {
				return;
			}
		}
		for ( int i = 0; i < paths.length; i++ ) {
			NodeInfo info = tree.getNodeInfo( paths[i] );
			if ( info.getType() != NodeInfo.Type.TABLE &&
				info.getType() != NodeInfo.Type.VIEW )
			{
				continue;
			}
			try {
				CsvSaver gen = new CsvSaver( param );
				gen.select( db, con, catalog, schema, info.getName() );
				if ( paths.length > 1 ) {
					gen.save( fc.getSelectedFile() );
				} else {
					gen.save( parent );
				}
			} catch ( Exception ex ) {
				error( ex, true );
			}
		}
	}

	protected void loadCsv() {
		TreePath[] paths = tree.getSelectionPaths();
		JFileChooser fc = null;
		if ( paths.length > 1 ) {
			fc = new JFileChooser( "." );
			fc.setFileSelectionMode( JFileChooser.DIRECTORIES_ONLY );
			int nRet = fc.showOpenDialog( parent );
			if ( nRet != JFileChooser.APPROVE_OPTION ) {
				return;
			}
		}
		for ( int i = 0; i < paths.length; i++ ) {
			NodeInfo info = tree.getNodeInfo( paths[i] );
			if ( info.getType() != NodeInfo.Type.TABLE &&
				info.getType() != NodeInfo.Type.VIEW )
			{
				continue;
			}
			try {
				CsvLoader gen = new CsvLoader( param );
				gen.select( db, con, catalog, schema, info.getName() );
				if ( paths.length > 1 ) {
					gen.load( fc.getSelectedFile() );
				} else {
					gen.load( parent );
				}
			} catch ( Exception ex ) {
				error( ex, true );
			}
		}
	}

	protected void generateJava() {
		TreePath[] paths = tree.getSelectionPaths();
		JFileChooser fc = null;
		if ( paths.length > 1 ) {
			fc = new JFileChooser( "." );
			fc.setFileSelectionMode( JFileChooser.DIRECTORIES_ONLY );
			int nRet = fc.showSaveDialog( parent );
			if ( nRet != JFileChooser.APPROVE_OPTION ) {
				return;
			}
		}
		for ( int i = 0; i < paths.length; i++ ) {
			NodeInfo info = tree.getNodeInfo( paths[i] );
			if ( info.getType() != NodeInfo.Type.TABLE &&
				info.getType() != NodeInfo.Type.VIEW )
			{
				continue;
			}
			try {
				IGenerator gen = GeneratorFactory.createJavaGenerator( param );
				gen.select( db, con, catalog, schema, info.getName() );
				if ( paths.length > 1 ) {
					gen.generate( fc.getSelectedFile() );
				} else {
					gen.generate( parent );
				}
			} catch ( Exception ex ) {
				error( ex, true );
			}
		}
	}

	protected void generateDocument() {
		TreePath[] paths = tree.getSelectionPaths();
		JFileChooser fc = null;
		if ( paths.length > 1 ) {
			fc = new JFileChooser( "." );
			fc.setFileSelectionMode( JFileChooser.DIRECTORIES_ONLY );
			int nRet = fc.showSaveDialog( parent );
			if ( nRet != JFileChooser.APPROVE_OPTION ) {
				return;
			}
		}
		for ( int i = 0; i < paths.length; i++ ) {
			NodeInfo info = tree.getNodeInfo( paths[i] );
			if ( info.getType() != NodeInfo.Type.TABLE &&
				info.getType() != NodeInfo.Type.VIEW )
			{
				continue;
			}
			try {
				IGenerator gen = GeneratorFactory.createDocumentGenerator( param );
				gen.select( db, con, catalog, schema, info.getName() );
				if ( paths.length > 1 ) {
					gen.generate( fc.getSelectedFile() );
				} else {
					gen.generate( parent );
				}
			} catch ( Exception ex ) {
				error( ex, true );
			}
		}
	}

	protected void treeDoubleClicked( MouseEvent ev ) {
		NodeInfo info = tree.getNodeInfo( ev.getX(), ev.getY() );
		if ( info == null || info.getLevel() != NodeInfo.Level.LEAF ) {
			return;
		}
		JInternalFrame panel = null;
		try {
			switch ( info.getType()) {
			case TABLE:
			case VIEW:
				showTableViewer( info );
				break;
			case PROCEDURE:
				showProcedureDesigner( info );
				break;
			}
		} catch ( Exception ex ) {
			error( ex, true );
			return;
		}
	}

	protected void showPopup( MouseEvent ev ) {
		selInfo = null;
		NodeInfo info = tree.getNodeInfo( ev.getX(), ev.getY() );
		if ( info == null ) {
			defMenu.show( tree, ev.getX(), ev.getY() );
			return;
		}
		TreePath path = tree.getPathForLocation( ev.getX(), ev.getY() );
		TreePath[] paths = tree.getSelectionPaths();
		if ( paths != null && paths.length > 0 ) {
			boolean found = false;
			for ( int i = 0; i < paths.length; i++ ) {
				if ( paths[i].equals( path )) {
					found = true;
					break;
				}
			}
			if ( !found ) {
				tree.setSelectionPath( path );
			}
		} else {
			tree.setSelectionPath( path );
		}
		if ( info.getLevel() == NodeInfo.Level.LEAF ) {
			selInfo = info;
			switch ( info.getType() ) {
			case TABLE:
			case VIEW:
				tblMenu.show( tree, ev.getX(), ev.getY() );
				break;
			case PROCEDURE:
				procMenu.show( tree, ev.getX(), ev.getY() );
				break;
			default:
				defMenu.show( tree, ev.getX(), ev.getY() );
			}
		} else
		if ( info.getLevel() == NodeInfo.Level.TYPE ) {
			JMenuItem itemCre = (JMenuItem )nodeMenu.getComponent( 0 );
			switch ( info.getType() ) {
			case TABLE:
				itemCre.setEnabled( true );
				nodeMenu.show( tree, ev.getX(), ev.getY() );
				break;
			case VIEW:
			case PROCEDURE:
				itemCre.setEnabled( false );
				nodeMenu.show( tree, ev.getX(), ev.getY() );
				break;
			}
		} else {
			defMenu.show( tree, ev.getX(), ev.getY() );
			return;
		}
	}

	public boolean start() {
		WindowUtility.center( parent, conDlg );
		boolean bRet = conDlg.start();
		if ( bRet ) {
			db = conDlg.getConnection();
			con = conDlg.getConnectInfo();
			con.setDisplayName( BrowserUtil.getConnectionName( con.getName() ));
			frmMng = new FrameManager( parent, param, db, con );
			frmMng.addTableEventListener( new TableEventListener() {
				public void tableChanged( TableEvent ev ) {
					refreshTree();
				}
			} );
			catalog = con.getCatalog();
			schema = con.getSchema();
			try {
				DatabaseMetaData meta = db.getConnection().getMetaData();
				selectSchema( meta );
				selectCatalog( meta );
				tree.select( db, con, catalog, schema );
				showFrame();
			} catch ( Exception ex ) {
				try {
					db.disconnect();
				} catch ( Exception de ) {
				}
				error( ex, false );
				return false;
			}
		}
		return bRet;
	}

	protected void selectSchema( DatabaseMetaData meta ) throws Exception {
		vecSch.clear();
		ResultSet rset = meta.getSchemas();
		for ( ;; ) {
			if ( !rset.next()) {
				break;
			}
			vecSch.add( rset.getString( "TABLE_SCHEM" ));
		}
	}

	protected void selectCatalog( DatabaseMetaData meta ) throws Exception {
		vecCat.clear();
		ResultSet rset = meta.getCatalogs();
		for ( ;; ) {
			if ( !rset.next()) {
				break;
			}
			vecCat.add( rset.getString( 1 ));
		}
	}

	private void showFrame() {
		DefaultComboBoxModel model;
		model = new DefaultComboBoxModel( vecCat );
		CMB_CAT.setModel( model );
		CMB_CAT.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				selectionChanged();
			}
		});
		model = new DefaultComboBoxModel( vecSch );
		CMB_SCH.setModel( model );
		CMB_SCH.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				selectionChanged();
			}
		});

		String title = prop.getString( "browser.title" );
		setTitle( title + " - " + con.getDisplayName() );
		frmMng.locatePanel( this );

		if ( catalog != null && catalog.length() > 0 ) {
			CMB_CAT.setSelectedItem( catalog );
		}
		if ( schema != null && schema.length() > 0 ) {
			CMB_SCH.setSelectedItem( schema );
		}
	}

	protected void selectionChanged() {
		catalog = (String )CMB_CAT.getSelectedItem();
		schema = (String )CMB_SCH.getSelectedItem();
		try {
			DatabaseMetaData meta = db.getConnection().getMetaData();
			tree.select( db, con, catalog, schema );
		} catch ( Exception ex ) {
			error( ex, true );
		}
	}

	public void closed() {
		frmMng.closeAll();
		frmMng.removePanel( this );
		BrowserUtil.removeConnectionName( con.getDisplayName() );
		stop();
	}
	public void stop() {
		try {
			db.disconnect();
			logger.sql( this, "disconnect " + db );
			db = null;
		} catch ( Exception ex ) {
		}
	}

}
