package	jp.co.sds_corp.jdbrowser.browser;

import java.sql.DatabaseMetaData;
import java.sql.ResultSet;

import javax.swing.JTree;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellEditor;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import jp.co.sds_corp.jdbrowser.NodeInfo;
import jp.co.sds_corp.jdbrowser.SetupParam;
import jp.co.sds_corp.jdbrowser.connect.ConnectInfo;
import jp.co.sds_corp.jdbrowser.util.BrowserUtil;
import jp.co.sds_corp.util.db.DbAccess;

/**
* DBのオブジェクトを表示します。
*/
public class BrowserTree extends JTree
{
	private static final long serialVersionUID = 1L;
	SetupParam	param;
	DbAccess	db;
	ConnectInfo	con;
	String		catalog;
	String		schema;
	BrowserTreeModel	model;
	DefaultMutableTreeNode	topNode;

	public BrowserTree( SetupParam param ) {
		this.param = param;
		setEditable( true );
		DefaultTreeCellEditor editor = (DefaultTreeCellEditor )getCellEditor();
		editor.addCellEditorListener( new EditorListener() );
	}

	protected NodeInfo getNodeInfo( TreePath path ) {
		DefaultMutableTreeNode node =
			(DefaultMutableTreeNode)path.getLastPathComponent();
		NodeInfo info = (NodeInfo )node.getUserObject();
		return info;
	}
	protected DefaultMutableTreeNode getNode( int x, int y ) {
		TreePath path = getPathForLocation( x, y );
		if ( path == null ) {
			return null;
		}
		DefaultMutableTreeNode node =
			(DefaultMutableTreeNode)path.getLastPathComponent();
		return node;
	}
	protected NodeInfo getNodeInfo( int x, int y ) {
		DefaultMutableTreeNode node = getNode( x, y );
		if ( node == null ) {
			return null;
		}
		NodeInfo info = (NodeInfo )node.getUserObject();
		return info;
	}

	public void select( DbAccess db, ConnectInfo con, String catalog, String schema )
		throws Exception
	{
		String name = BrowserUtil.getName( catalog, schema );
		NodeInfo info = new NodeInfo( name );
		topNode = new DefaultMutableTreeNode( info );
		model = new BrowserTreeModel( topNode );
		setModel( model );

		this.db = db;
		this.con = con;
		this.catalog = catalog;
		this.schema = schema;

		DatabaseMetaData meta = db.getConnection().getMetaData();
		selectTable( meta );
		selectView( meta );
		selectProcedure( meta );
		model.reload();
		expandAll();
	}

	protected void selectTable( DatabaseMetaData meta )
		throws Exception
	{
		NodeInfo info = new NodeInfo( NodeInfo.Type.TABLE, "Table" );
		DefaultMutableTreeNode nodeType = new DefaultMutableTreeNode( info ); 
		topNode.add( nodeType );

		String[] types = { "TABLE", "" };
		if ( param.isIncludeSystem()) {
			types[1] = "SYSTEM TABLE";
		}
		select( meta, types, nodeType, NodeInfo.Type.TABLE );
	}

	protected void selectView( DatabaseMetaData meta )
		throws Exception
	{
		NodeInfo info = new NodeInfo( NodeInfo.Type.VIEW, "View" );
		DefaultMutableTreeNode nodeType = new DefaultMutableTreeNode( info ); 
		topNode.add( nodeType );

		String[] types = { "VIEW", "" };
		if ( param.isIncludeSystem()) {
			types[1] = "SYSTEM VIEW";
		}
		select( meta, types, nodeType, NodeInfo.Type.VIEW );
	}

	protected void select( DatabaseMetaData meta, String[] types,
						DefaultMutableTreeNode nodeType, NodeInfo.Type type )
		throws Exception
	{
		ResultSet rset = meta.getTables( catalog, schema, null, types );
		for ( ;; ) {
			if ( !rset.next()) {
				break;
			}
			NodeInfo info = new NodeInfo( type, con,
								rset.getString( "TABLE_CAT" ),
								rset.getString( "TABLE_SCHEM" ),
								rset.getString( "TABLE_NAME" ) );
			DefaultMutableTreeNode node = new DefaultMutableTreeNode( info ); 
			nodeType.add( node );
		}
	}

	protected void selectIndex( DatabaseMetaData meta, String table )
		throws Exception
	{
		NodeInfo info = new NodeInfo( NodeInfo.Type.INDEX, "Index" );
		DefaultMutableTreeNode nodeType = new DefaultMutableTreeNode( info ); 
		topNode.add( nodeType );

		ResultSet rset = meta.getIndexInfo( catalog, schema, table, false, false );
		for ( ;; ) {
			if ( !rset.next()) {
				break;
			}
			info = new NodeInfo( NodeInfo.Type.INDEX, con,
								rset.getString( "TABLE_CAT" ),
								rset.getString( "TABLE_SCHEM" ),
								rset.getString( "TABLE_NAME" ) + "."
								+ rset.getString( "INDEX_NAME" ) );
			DefaultMutableTreeNode node = new DefaultMutableTreeNode( info ); 
			nodeType.add( node );
		}
	}

	protected void selectProcedure( DatabaseMetaData meta )
		throws Exception
	{
		NodeInfo info = new NodeInfo( NodeInfo.Type.PROCEDURE, "Procedure" );
		DefaultMutableTreeNode nodeType = new DefaultMutableTreeNode( info ); 
		topNode.add( nodeType );

		ResultSet rset = meta.getProcedures( catalog, schema, null );
		for ( ;; ) {
			if ( !rset.next()) {
				break;
			}
			info = new NodeInfo( NodeInfo.Type.PROCEDURE, con,
								rset.getString( "PROCEDURE_CAT" ),
								rset.getString( "PROCEDURE_SCHEM" ),
								rset.getString( "PROCEDURE_NAME" ) );
			DefaultMutableTreeNode node = new DefaultMutableTreeNode( info ); 
			nodeType.add( node );
		}
	}

	class EditorListener implements CellEditorListener {
		public void editingCanceled( ChangeEvent ev ) {
		}
		public void editingStopped( ChangeEvent ev ) {
			DefaultTreeCellEditor editor =
				(DefaultTreeCellEditor )getCellEditor();
			try {
				renameTo( (String )editor.getCellEditorValue() );
			} catch ( Exception ex ) {
				try {
					db.rollback();
				} catch ( Exception ex2 ) {
				}
			}
			editor.cancelCellEditing();
		}
	}

	protected void renameTo( String newName ) 
		throws Exception
	{
		NodeInfo info = getNodeInfo( getSelectionPath() );
		if ( info.getName().equals( newName )) {
			return;
		}
		String sql;
		sql = "ALTER TABLE " + info.getName() + " RENAME TO " + newName;
		db.execute( sql );
		db.commit();
		info.setName( newName );
	}

	public void removeAll() {
		TreePath[] all = getPathBetweenRows( 0, getRowCount() );
		removeSelectionPaths( all );
		model.reload();
	}

	public void expandAll() {
		for ( int i = 0; i < getRowCount(); i++ ) {
			expandRow( i );
		}
	}

	class BrowserTreeModel extends DefaultTreeModel
	{
		private static final long serialVersionUID = 1L;

		BrowserTreeModel( TreeNode node ) {
			super( node );
		}
		public boolean isLeaf( Object obj ) {
			DefaultMutableTreeNode node = (DefaultMutableTreeNode )obj;
			NodeInfo info = (NodeInfo )node.getUserObject();
			if ( info.getLevel() != NodeInfo.Level.LEAF ) {
				return false;
			}
			return true;
		}
	}

}
