package	jp.co.sds_corp.jdbrowser.connect;

import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import jp.co.sds_corp.jdbrowser.SetupParam;
import jp.co.sds_corp.util.common.CmnProperty;
import jp.co.sds_corp.util.common.JarClassLoader;
import jp.co.sds_corp.util.db.DbAccess;
import jp.co.sds_corp.util.db.DbInfo;
import jp.co.sds_corp.util.log.CmnLogger;
import jp.co.sds_corp.util.window.ValuePanel;
import jp.co.sds_corp.util.window.WindowUtility;

/**
* DB接続情報を設定します。
*/
public class ConnectDialog extends JDialog
{
	private static final long serialVersionUID = 1L;
	static CmnLogger logger = CmnLogger.getInstance();
	SetupParam	param;
	JScrollPane sp;
	JList	list;
	JButton	BTN_ADD;
	JButton	BTN_DEL;
	ConnectListModel model = new ConnectListModel();
	ValuePanel	VAL_NAME = new ValuePanel( ValuePanel.ReturnType.STRING, "Name:" );
	ValuePanel	VAL_JAR = new ValuePanel( ValuePanel.ReturnType.STRING, "Jar:" );
	ValuePanel	VAL_DRV = new ValuePanel( ValuePanel.ReturnType.STRING, "Driver:" );
	ValuePanel	VAL_URL = new ValuePanel( ValuePanel.ReturnType.STRING, "URL:" );
	ValuePanel	VAL_USR = new ValuePanel( ValuePanel.ReturnType.STRING, "User:" );
	ValuePanel	VAL_PWD = new ValuePanel( ValuePanel.ReturnType.STRING, ValuePanel.Type.PASSWORD, "Password:" );
	ValuePanel	VAL_ENC = new ValuePanel( ValuePanel.ReturnType.STRING, "Encode:" );
	ValuePanel	VAL_CAT = new ValuePanel( ValuePanel.ReturnType.STRING, "Catalog:" );
	ValuePanel	VAL_SCH = new ValuePanel( ValuePanel.ReturnType.STRING, "Schema:" );
	JButton		BTN_OK;
	JButton		BTN_CANCEL;
	int			nRes = -1;
	int			selIdx = -1;
	boolean		bCheck = true;
	boolean		bAdd = false;
	ConnectInfo	selInfo, conInfo;
	DbAccess	db = null;
	JarClassLoader	loader;
	CmnProperty	prop;

	public ConnectDialog( SetupParam param ) {
		this.param = param;
		prop = new CmnProperty( "resource.jdbrowser" );
		createControls();
	}

	protected void createControls() {
		setModal( true );
		//setResizable( false );
		setSize( 400, 280 );
		JPanel panel = new JPanel();
		getContentPane().setLayout(
			new BoxLayout( getContentPane(), BoxLayout.Y_AXIS ));
		WindowUtility.setHideOnEscape( this );

		Box panel1 = new Box( BoxLayout.X_AXIS );
		panel1.add( createListPanel() );
		panel1.add( Box.createHorizontalStrut( 10 ));
		panel1.add( createInputPanel() );
		getContentPane().add( panel1 );
		getContentPane().add( createButtonPanel() );

		String title = prop.getString( "connect.title" );
		setTitle( title );
	}

	private JComponent createListPanel() {
		Box panel = new Box( BoxLayout.Y_AXIS );
		list = new JList( model );
		list.setVisibleRowCount( 10 );
		list.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		list.addListSelectionListener( new ListSelectionListener() {
			public void valueChanged( ListSelectionEvent ev ) {
				int idx = list.getSelectedIndex();
				if ( bCheck && !ev.getValueIsAdjusting() && idx >= 0 ) {
					selected( idx );
				}
				bCheck = true;
			}
		});
		list.addMouseListener( new MouseAdapter() {
			public void mouseClicked( MouseEvent ev ) {
				if ( ev.getClickCount() == 2 ) {
					onOk();
				}
			}
		} );

		Vector<ConnectInfo> vec = param.listConnection();
		for ( int i = 0; i < vec.size(); i++ ) {
			ConnectInfo info = (ConnectInfo )vec.get( i );
			model.addElement( info.getName() );
		}
		if ( param.listConnection().size() > 0 ) {
			list.setSelectedIndex( 0 );
		}
		sp = new JScrollPane( list,
			JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
			JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );
		panel.add( sp );

		Box btnPnl = new Box( BoxLayout.X_AXIS );
		String text;
		BTN_ADD = new JButton();
		WindowUtility.setButtonIcon( this, BTN_ADD, "/resource/plus.png" );
		BTN_ADD.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				addCon();
			}
		} );
		text = prop.getString( "connect.add.tooltip" );
		BTN_ADD.setToolTipText( text );
		BTN_ADD.setMargin( new Insets( 2, 2, 2, 2 ));
		btnPnl.add( BTN_ADD );

		BTN_DEL = new JButton();
		WindowUtility.setButtonIcon( this, BTN_DEL, "/resource/minus.png" );
		BTN_DEL.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				delCon();
			}
		} );
		text = prop.getString( "connect.del.tooltip" );
		BTN_DEL.setToolTipText( text );
		BTN_DEL.setMargin( new Insets( 2, 2, 2, 2 ));
		btnPnl.add( BTN_DEL );

		panel.add( btnPnl );
		return panel;
	}

	private JPanel createButtonPanel() {
		JPanel panel = new JPanel();
		BTN_OK = new JButton( "OK" );
		BTN_OK.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				onOk();
			}
		});
		panel.add( BTN_OK );
		getRootPane().setDefaultButton( BTN_OK );
		BTN_CANCEL = new JButton( "Cancel" );
		BTN_CANCEL.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				onCancel();
			}
		});
		panel.add( BTN_CANCEL );
		return panel;
	}

	private JComponent createInputPanel() {
		Box panel = new Box( BoxLayout.Y_AXIS );
		panel.add( VAL_NAME );
		panel.add( VAL_JAR );
		panel.add( VAL_DRV );
		panel.add( VAL_URL );
		panel.add( VAL_USR );
		panel.add( VAL_PWD );
		panel.add( VAL_ENC );
		panel.add( VAL_CAT );
		panel.add( VAL_SCH );
		return panel;
	}

	protected void addCon() {
		try {
			updateConnectInfo( selInfo );
		} catch ( Exception ex ) {
			String title = prop.getString( "connect.err2.title" );
			String msg = prop.getString( "connect.err2.message" );
			JOptionPane.showMessageDialog( this,
				msg + "\n" + ex, title,
				JOptionPane.ERROR_MESSAGE );
			return;
		}
		VAL_NAME.setValue( "" );
		VAL_JAR.setValue( "" );
		VAL_DRV.setValue( "" );
		VAL_URL.setValue( "" );
		VAL_USR.setValue( "" );
		VAL_PWD.setValue( "" );
		VAL_ENC.setValue( "" );
		VAL_CAT.setValue( "" );
		VAL_SCH.setValue( "" );

		bCheck = false;
		selIdx = model.getSize();
		model.addElement( " " );
		selInfo = new ConnectInfo();
		list.setSelectedIndex( selIdx );
		list.ensureIndexIsVisible( selIdx );
		VAL_NAME.requestFocus();
		bAdd = true;
	}

	protected void delCon() {
		if ( selIdx < 0 ) {
			return;
		}
		if ( bAdd ) {
			model.removeElementAt( selIdx );
			selInfo = null;
			list.setSelectedIndex( model.getSize() - 1 );
		} else {
			String name = (String )model.remove( selIdx );
			param.mapConnection().remove( name );
			param.listConnection().remove( selIdx );
			if ( selIdx >= model.getSize()) {
				if ( model.getSize() == 0 ) {
					selIdx = -1;
				}
				list.setSelectedIndex( model.getSize() - 1 );
			} else {
				list.setSelectedIndex( selIdx );
			}
		}
		bAdd = false;
	}

	private DbInfo getDbInfo( ConnectInfo info ) {
		DbInfo ret = new DbInfo( info.getDriver(), info.getUrl(),
			info.getUser(), info.getPassword(), info.getEncode() );
		return ret;
	}

	private void updateConnectInfo( ConnectInfo info ) throws Exception {
		String name = (String )VAL_NAME.getValue();
		if ( name == null || name.length() == 0 ) {
			VAL_NAME.requestFocus();
			throw( new Exception( "must input name" ));
		}
		String driver = (String )VAL_DRV.getValue();
		if ( driver == null || driver.length() == 0 ) {
			VAL_DRV.requestFocus();
			throw( new Exception( "must input driver" ));
		}
		String url = (String )VAL_URL.getValue();
		if ( url == null || url.length() == 0 ) {
			VAL_URL.requestFocus();
			throw( new Exception( "must input URL" ));
		}

		if ( !name.equals( info.getName())) {
			if ( param.mapConnection().get( name ) != null ) {
				VAL_NAME.requestFocus();
				throw( new Exception( "name duplicated" ));
			}
			if ( bAdd ) {
				param.mapConnection().put( name, info );
				int idx = model.getSize() - 1;
				model.setElementAt( name, idx );
				param.listConnection().add( info );
			} else {
				param.mapConnection().remove( info.getName() );
				param.mapConnection().put( name, info );
				int idx = model.indexOf( info.getName() );
				model.setElementAt( name, idx );
			}
			info.setName( name );
		}
		info.setJar( (String )VAL_JAR.getValue() );
		info.setDriver( driver );
		info.setUrl( url );
		info.setUser( (String )VAL_USR.getValue() );
		info.setPassword( (String )VAL_PWD.getValue() );
		String enc = (String )VAL_ENC.getValue();
		if ( enc != null && enc.length() == 0 ) {
			enc = null;
		}
		info.setEncode( enc );

		String catalog = (String )VAL_CAT.getValue();
		if ( catalog != null && catalog.length() == 0 ) {
			catalog = null;
		}
		info.setCatalog( catalog );
		String schema = (String )VAL_SCH.getValue();
		if ( schema != null && schema.length() == 0 ) {
			schema = null;
		}
		info.setSchema( schema );
	}

	public void onOk() {
		nRes = -1;
		if ( selIdx < 0 ) {
			return;
		}
		try {
			updateConnectInfo( selInfo );
		} catch ( Exception ex ) {
			String title = prop.getString( "connect.err2.title" );
			String msg = prop.getString( "connect.err2.message" );
			JOptionPane.showMessageDialog( this,
				msg + "\n" + ex, title,
				JOptionPane.ERROR_MESSAGE );
			return;
		}
		try {
			DbInfo info = getDbInfo( selInfo );
			java.sql.Driver drv = null;
			db = new DbAccess( info );
			if ( selInfo.getJar() != null && selInfo.getJar().length() > 0 ) {
				loader = new JarClassLoader( selInfo.getJar(),
										getClass().getClassLoader());
				Class cl = loader.loadClass( selInfo.getDriver() );
				drv = (java.sql.Driver )cl.newInstance();
				db.connect( drv, selInfo.getProperties() );
			} else {
				db.connect( selInfo.getProperties() );
			}
			conInfo = (ConnectInfo )selInfo.clone();
			conInfo.setDbInfo( info );
			if ( drv != null ) {
				conInfo.setSqlDriver( drv );
			}

			setVisible( false );
		} catch ( Throwable ex ) {
			try {
				db.disconnect();
			} catch ( Exception de ) {
			}
			db = null;
			logger.error( this, "connect error" + ex );
			if ( param.isDebug()) {
				ex.printStackTrace();
			}
			String title = prop.getString( "connect.err1.title" );
			String msg = prop.getString( "connect.err1.message" );
			JOptionPane.showMessageDialog( this,
				msg + "\n" + ex, title,
				JOptionPane.ERROR_MESSAGE );
			return;
		}
		nRes = 0;
	}

	public void onCancel() {
		nRes = 1;
		setVisible( false );
	}
	public int getResult() {
		return nRes;
	}

	public void selected( int idx ) {
		bCheck = false;
		if ( selInfo != null ) {
			try {
				updateConnectInfo( selInfo );
			} catch ( Exception ex ) {
				list.setSelectedIndex( selIdx );
				String title = prop.getString( "connect.err2.title" );
				String msg = prop.getString( "connect.err2.message" );
				JOptionPane.showMessageDialog( this,
					msg + "\n" + ex, title,
					JOptionPane.ERROR_MESSAGE );
				return;
			}
		}
		String name = (String )model.get( idx );
		selInfo = (ConnectInfo )param.mapConnection().get( name );
		VAL_NAME.setValue( name );
		VAL_JAR.setValue( selInfo.getJar() );
		VAL_DRV.setValue( selInfo.getDriver() );
		VAL_URL.setValue( selInfo.getUrl() );
		VAL_USR.setValue( selInfo.getUser() );
		VAL_PWD.setValue( selInfo.getPassword() );
		VAL_ENC.setValue( selInfo.getEncode() );
		VAL_CAT.setValue( selInfo.getCatalog() );
		VAL_SCH.setValue( selInfo.getSchema() );
		selIdx = idx;
		bAdd = false;
	}

	public boolean start() {
		setVisible( true );
		if ( nRes != 0 ) {
			return false;
		}
		return true;
	}

	public void stop() {
	}

	public DbAccess getConnection() {
		return db;
	}

	public ConnectInfo getConnectInfo() {
		return conInfo;
	}

	public class ConnectListModel extends DefaultListModel
	{
	}

}
