package	jp.co.sds_corp.jdbrowser.generator;

import java.awt.Window;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.sql.PreparedStatement;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import jp.co.sds_corp.jdbrowser.SetupParam;
import jp.co.sds_corp.util.db.DbAccess;
import jp.co.sds_corp.util.db.DbException;
import jp.co.sds_corp.util.db.DbTable;
import jp.co.sds_corp.util.log.CmnLogger;
import jp.co.sds_corp.util.window.ExtFileFilter;

/**
<pre>
&lt;CSV
	columnSeparator="\t"
	rowSeparator="\n"
	encode="EUC-JP" /&gt;
</pre>
*/
public class CsvLoader extends AbstractGenerator
{
	static CmnLogger logger = CmnLogger.getInstance();
	CsvParam	cparam;

	public CsvLoader( SetupParam param )
		throws Exception
	{
		super( param );
		cparam = param.getCsvParam();
	}

	public void load( Window parent ) throws Exception {
		String file = selectFile( parent, table );
		load( file );
	}

	public String selectFile( Window parent, String table ) {
		return selectFile( parent, table, null );
	}

	public String selectFile( Window parent, String table, String defDir ) {
		String[] ext = { "bcp" };
		String text = prop.getString( "csv.load.filter" );
		if ( defDir == null ) {
			defDir = ".";
		}
		JFileChooser fc = new JFileChooser( defDir ) {
			private static final long serialVersionUID = 1L;

			public void approveSelection() {
				File sel = getSelectedFile();
				if ( getFileFilter() instanceof ExtFileFilter ) {
					if ( !sel.getPath().toLowerCase().endsWith( ".bcp" )) {
						sel = new File( sel.getPath() + ".bcp" );
						setSelectedFile( sel );
					}
				}
				if ( !sel.exists() ) {
					String title = prop.getString( "csv.load.err.title" );
					String msg = prop.getString( "csv.load.err.message" );
					JOptionPane.showMessageDialog( this,
						msg, title, JOptionPane.ERROR_MESSAGE );
					return;
				}
				super.approveSelection();
			}
		};
		fc.setSelectedFile( new File( table + ".bcp" ));
		fc.setFileFilter( new ExtFileFilter( text, ext ) );
		int nRet = fc.showOpenDialog( parent );
		if ( nRet != JFileChooser.APPROVE_OPTION ) {
			return null;
		}
		return fc.getSelectedFile().toString();
	}

	public void load( File path ) throws Exception {
		String file = path.toString() + File.separator
						+ table + ".bcp";
		load( file );
	}

	public void load( String file ) throws Exception {
		String sql = "select * from " + table + " where 1=0";
		DbTable tbl = new DbTable();
		try {
			db.executeQuery( sql );
			db.fetch( tbl );
		} catch ( DbException de ) {
			System.out.println( table + " not found." );
			return;
		}
		db.cancel();

		sql = "insert into " + table + " values (";
		for ( int i = 0; i < tbl.getColumnCount(); i++ ) {
			if ( i > 0 ) {
				sql += ",";
			}
			sql += "?";
		}
		sql += ")";
		db.prepareQuery( sql );

		BufferedReader br = new BufferedReader(
			new InputStreamReader( new FileInputStream( file )));
		int nRow = 0;
		for ( ;; ) {
			String line = br.readLine();
			if ( line == null ) {
				break;
			}
			nRow++;
			try {
				tbl.parse( line );
			} catch ( Exception ex ) {
				br.close();
				db.rollback();
				db.cancel();
				throw( new Exception( "text parse error: line=" + nRow
									+ "[" + line + "]" + ex.getMessage(),
									ex ));
			}
			insert( db, tbl );
		}
		br.close();
		db.cancel();
		if ( param.isDebug()) {
			logger.debug( this, "(" + nRow + " rows loaded)" );
		}
	}

	private void insert( DbAccess db, DbTable tbl )
		throws Exception
	{
		PreparedStatement st = db.getStatement();
		for ( int i = 0; i < tbl.getColumnCount(); i++ ) {
			st.setObject( i + 1, tbl.getObject( i ));
		}
		db.executePrepared();
	}
}
