package	jp.co.sds_corp.jdbrowser.generator;

import java.awt.Window;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import jp.co.sds_corp.jdbrowser.SetupParam;
import jp.co.sds_corp.util.db.DbTable;
import jp.co.sds_corp.util.window.ExtFileFilter;

/**
<pre>
&lt;CSV
	columnSeparator="\t"
	rowSeparator="\n"
	encode="EUC-JP" /&gt;
</pre>
*/

public class CsvSaver extends AbstractGenerator
{
	CsvParam	cparam;

	public CsvSaver( SetupParam param )
		throws Exception
	{
		super( param );
		cparam = param.getCsvParam();
	}

	public void save( Window parent ) throws Exception {
		String file = selectFile( parent, table );
		save( file );
	}

	public String selectFile( Window parent, String table ) {
		return selectFile( parent, table, null );
	}

	public String selectFile( Window parent, String table, String defDir ) {
		String[] ext = { "bcp" };
		String text = prop.getString( "csv.save.filter" );
		if ( defDir == null ) {
			defDir = ".";
		}
		JFileChooser fc = new JFileChooser( defDir ) {
			private static final long serialVersionUID = 1L;

			public void approveSelection() {
				File sel = getSelectedFile();
				if ( getFileFilter() instanceof ExtFileFilter ) {
					if ( !sel.getPath().toLowerCase().endsWith( ".bcp" )) {
						sel = new File( sel.getPath() + ".bcp" );
						setSelectedFile( sel );
					}
				}
				if ( sel.exists() ) {
					String title = prop.getString( "csv.save.exists.title" );
					String msg = prop.getString( "csv.save.exists.message" );
					int nRet = JOptionPane.showConfirmDialog( this,
						msg, title,
						JOptionPane.YES_NO_OPTION );
					if ( nRet != JOptionPane.YES_OPTION ) {
						return;
					}
				}
				super.approveSelection();
			}
		};
		fc.setSelectedFile( new File( table + ".bcp" ));
		fc.setFileFilter( new ExtFileFilter( text, ext ) );
		int nRet = fc.showSaveDialog( parent );
		if ( nRet != JFileChooser.APPROVE_OPTION ) {
			return null;
		}
		return fc.getSelectedFile().toString();
	}

	public void save( File path ) throws Exception {
		String file = path.toString() + File.separator
						+ table + ".bcp";
		save( file );
	}

	public void save( String file ) throws Exception {
		PrintWriter out;
		if ( cparam.getEncode() == null ) {
			out = new PrintWriter( new FileWriter( file ));
		} else {
			out = new PrintWriter( new OutputStreamWriter(
				new FileOutputStream( file ), cparam.getEncode() ));
		}
		String sql = "SELECT * FROM " + table;
		DbTable tbl = new DbTable();
		db.executeQuery( sql );
		for ( ;; ) {
			if ( !db.fetch( tbl )) {
				break;
			}
			for ( int i = 0; i < tbl.getColumnCount(); i++ ) {
				if ( i > 0 ) {
					out.print( cparam.getColumnSeparator() );
				}
				String dat = tbl.getString( i );
				if ( dat != null ) {
					out.print( dat );
				}
			}
			out.print( cparam.getRowSeparator() );
		}
		out.flush();
		out.close();
	}

}
