package	jp.co.sds_corp.jdbrowser.generator;

import java.awt.Window;
import java.io.File;
import java.io.FileOutputStream;
import java.util.Vector;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import jp.co.sds_corp.jdbrowser.ColumnInfo;
import jp.co.sds_corp.jdbrowser.SetupParam;
import jp.co.sds_corp.report.IReport;
import jp.co.sds_corp.report.ReportFactory;
import jp.co.sds_corp.util.window.ExtFileFilter;

/**
詳細は、{@link jp.co.sds_corp.report.ReportFactory}を参照してください。
<pre>
&lt;Document
	generator="foo.bar"
	type="Excel"
	template="table.xls" sheet="Sheet1"&gt;
	&lt;Script language="JavaScript"&gt;
		Report.setCell( "E4", bean.table );
		for ( i = 0; i &lt; bean.list.size(); i++ ) {
			var data = bean.list.get( i );
			Report.addRow( 6+i, 6 );
			Report.setCell( 3, 6+i, data.name );
		}
	&lt;/Script&gt;
&lt;/Document&gt;
</pre>
*/
public class DocumentGenerator extends AbstractGenerator implements IGenerator
{
	DocParam	dparam;
	IReport	report;

	public DocumentGenerator( SetupParam param )
		throws Exception
	{
		super( param );
		dparam = param.getDocParam();
		report = ReportFactory.createReport( param.getDocParam().getConfiguration() );
	}

	public void generate( Window parent ) throws Exception {
		String[] ext = { "xls" };
		String text = prop.getString( "doc.filter" );
		JFileChooser fc = new JFileChooser( "." ) {
			private static final long serialVersionUID = 1L;

			public void approveSelection() {
				File sel = getSelectedFile();
				if ( getFileFilter() instanceof ExtFileFilter ) {
					if ( !sel.getPath().toLowerCase().endsWith( ".xls" )) {
						sel = new File( sel.getPath() + ".xls" );
						setSelectedFile( sel );
					}
				}
				if ( sel.exists() ) {
					String title = prop.getString( "doc.title" );
					String msg = prop.getString( "doc.message" );
					int nRet = JOptionPane.showConfirmDialog( this,
						msg, title,
						JOptionPane.YES_NO_OPTION );
					if ( nRet != JOptionPane.YES_OPTION ) {
						return;
					}
				}
				super.approveSelection();
			}
		};
		fc.setSelectedFile( new File( table + ".xls" ));
		fc.setFileFilter( new ExtFileFilter( text, ext ) );
		int nRet = fc.showSaveDialog( parent );
		if ( nRet != JFileChooser.APPROVE_OPTION ) {
			return;
		}
		generate( fc.getSelectedFile().toString() );
	}

	public void generate( File path ) throws Exception {
		String file = path.toString() + File.separator
						+ table + ".xls";
		generate( file );
	}
	public void generate( String file ) throws Exception {
		IReport report;
		report = ReportFactory.createReport( param.getDocParam().getConfiguration() );
		report.setAttribute( "Caller", this );
		Bean bean = new Bean();
		bean.setTable( table );
		bean.getList().addAll( vecClm );
		report.setAttribute( "bean", bean );
		FileOutputStream out = new FileOutputStream( file );
		report.print( out );
		out.flush();
		out.close();
	}

	public class Bean
	{
		String	table;	// テーブル名
		Vector<ColumnInfo>	list = new Vector<ColumnInfo>();	// データ一覧

		/**
		* テーブル名を設定します。
		* @param	var		設定するデータ
		*/
		public void setTable( String var ) {
			this.table = var;
		}
		/**
		* テーブル名を取得します。
		* @return	テーブル名
		*/
		public String getTable() {
			return table;
		}

		/**
		* データ一覧を設定します。
		* @param	var		設定するデータ
		*/
		public void setList( Vector<ColumnInfo> var ) {
			this.list = var;
		}
		/**
		* データ一覧を取得します。
		* @return	データ一覧
		*/
		public Vector<ColumnInfo> getList() {
			return list;
		}


	}

}
