package	jp.co.sds_corp.jdbrowser.generator;

import java.awt.Window;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import jp.co.sds_corp.jdbrowser.ColumnInfo;
import jp.co.sds_corp.jdbrowser.SetupParam;
import jp.co.sds_corp.util.window.ExtFileFilter;

/**
<pre>
&lt;Java
	package="jp.co.sds_corp.foo"
	util_package="jp.co.sds_corp.util"
	generator="foo.bar"
	encode="EUC-JP"&gt;
	&lt;audit_column column="update_date"/&gt;
&lt;/Java&gt;
</pre>
*/
public class JavaGenerator extends AbstractGenerator implements IGenerator
{
	JavaParam jparam;

	public JavaGenerator( SetupParam param )
		throws Exception
	{
		super( param );
		jparam = param.getJavaParam();
	}

	public void generate( Window parent ) throws Exception {
		String[] ext = { "java" };
		String text = prop.getString( "java.filter" );
		JFileChooser fc = new JFileChooser( "." ) {
			private static final long serialVersionUID = 1L;

			public void approveSelection() {
				File sel = getSelectedFile();
				if ( getFileFilter() instanceof ExtFileFilter ) {
					if ( !sel.getPath().toLowerCase().endsWith( ".java" )) {
						sel = new File( sel.getPath() + ".java" );
						setSelectedFile( sel );
					}
				}
				if ( sel.exists() ) {
					String title = prop.getString( "java.title" );
					String msg = prop.getString( "java.message" );
					int nRet = JOptionPane.showConfirmDialog( this,
						msg, title,
						JOptionPane.YES_NO_OPTION );
					if ( nRet != JOptionPane.YES_OPTION ) {
						return;
					}
				}
				super.approveSelection();
			}
		};
		fc.setSelectedFile( new File( getClassName() + ".java" ));
		fc.setFileFilter( new ExtFileFilter( text, ext ) );
		int nRet = fc.showSaveDialog( parent );
		if ( nRet != JFileChooser.APPROVE_OPTION ) {
			return;
		}
		generate( fc.getSelectedFile().toString() );
	}

	public void generate( File path ) throws Exception {
		String file = path.toString() + File.separator
						+ getClassName() + ".java";
		generate( file );
	}
	public void generate( String file ) throws Exception {
		PrintWriter out;
		if ( jparam.getEncode() == null ) {
			out = new PrintWriter( new FileWriter( file ));
		} else {
			out = new PrintWriter( new OutputStreamWriter(
				new FileOutputStream( file ), jparam.getEncode() ));
		}
		printHeader( out );
		for ( int i = 0; i < vecClm.size(); i++ ) {
			printColumn( out, (ColumnInfo )vecClm.get( i ), i );
		}
		printFooter( out );
		out.flush();
		out.close();
	}

	protected String getClassName() {
		String[] arr = table.split( "_" );
		StringBuffer buf = new StringBuffer();
		for ( int i = 0; i < arr.length; i++ ) {
			buf.append( arr[i].substring( 0, 1 ).toUpperCase() );
			buf.append( arr[i].substring( 1 ).toLowerCase() );
		}
		return buf.toString();
	}

	protected void printHeader( PrintWriter out ) {
		out.println( "package " + jparam.getPackage() + ";" );
		out.println( "" );
		out.println( "import\tjava.math.*;" );
		out.println( "import\tjava.sql.*;" );
		out.println( "" );
		out.println( "import\t" + jparam.getUtilPackage() + ".log.*;" );
		out.println( "import\t" + jparam.getUtilPackage() + ".db.*;" );
		out.println( "" );
		out.println( "/**" );
		out.println( "* " + table + "テーブルのI/Oクラス" );
		out.println( "* @see DbTable" );
		out.println( "*/" );
		out.println( "public class " + getClassName() + " extends DbTable" );
		out.println( "{" );
		printConstructor( out );
	}

	protected void printConstructor( PrintWriter out ) {
		out.println( "\t/**" );
		out.println( "\t* 新しい " + getClassName() + " オブジェクトを初期化します" );
		out.println( "\t* @see DbTable" );
		out.println( "\t*/" );
		out.println( "\tpublic " + getClassName() + "() {" );
		out.println( "\t\tsuper( \"" + table + "\" );" );
		for ( int i = 0; i < jparam.listAuditColumn().size(); i++ ) {
			String clm = (String )jparam.listAuditColumn().get( i );
			out.println( "\t\taddAuditColumn( \"" + clm + "\" );" );
		}
		out.println( "\t\tsetInitColumn( true );" );

		for ( int i = 0; i < vecClm.size(); i++ ) {
			ColumnInfo col = (ColumnInfo )vecClm.get( i );
			out.println( "\t\taddColumn( new DbColumn( \"" + col.getName()
					+ "\", null, " + col.getSqlType() + ", "
					+ col.getSize() + ", "
					+ col.getPrecision() + ", "
					+ col.getScale() + ", 0 ));" );
		}
		out.println( "\t}" );
		out.println( "" );
	}

	protected void printColumn( PrintWriter out, ColumnInfo col, int nPos ) {
		out.println( "\t/**" );
		if ( col.getRemarks() == null ) {
			out.println( "\t* " + col.getName() + "のデータを設定します" );
		} else {
			out.println( "\t* " + col.getRemarks()
						+ "(" + col.getName() + ")のデータを設定します" );
		}
		out.println( "\t* @param\tvar\t\t設定するデータ" );
		out.println( "\t* @return\tなし" );
		out.println( "\t*/" );
		out.println( "\tpublic void set" + col.getMethodName() + "( "
						+ col.getFetchType() + " var ) {" );
		out.println( "\t\tsetObject( \"" + col.getName() + "\", var );" );
		out.println( "\t}" );
		out.println( "\t/**" );
		if ( col.getRemarks() == null ) {
			out.println( "\t* " + col.getName() + "のデータを取得します" );
		} else {
			out.println( "\t* " + col.getRemarks()
						+ "(" + col.getName() + ")のデータを取得します" );
		}
		out.println( "\t* @return\t" + col.getName()  + "の内容" );
		out.println( "\t*/" );
		out.println( "\tpublic " + col.getFetchType()
						+ " get" + col.getMethodName() + "() {" );
		out.println( "\t\treturn (" + col.getFetchType()
						+ " )getObject( \"" + col.getName() + "\" );" );
		out.println( "\t}" );
		out.println( "" );
	}

	protected void printFooter( PrintWriter out ) {
		printSelectSql( out );
		printSelectByKeySql( out );
		printInsertSql( out );
		printDeleteSql( out );
		printDebug( out );
		out.println( "}" );
	}

	protected void printSelectSql( PrintWriter out ) {
		out.println( "\t/**" );
		out.println( "\t* " + table + "テーブルから全カラムを取得するSQL文を作成し、実行します" );
		out.println( "\t* @param\t\twhere where/order by/group by句" );
		out.println( "\t* @return\tSQL文" );
		out.println( "\t*/" );
		out.println( "\tpublic String getSelectSql( String where ) {" );
		out.println( "\t\tString  sql =" );
		out.println( "\t\t\t\"select \"" );

		out.print( getAllColumn() );

		out.println( "\t\t\t+ \" from " + table + " \";" );
		out.println( "\t\tif ( where != null ) {" );
		out.println( "\t\t\tsql += where;" );
		out.println( "\t\t}" );
		out.println( "\t\treturn sql;" );
		out.println( "\t}" );
		out.println( "" );
	}

	private String getAllColumn() {
		StringBuffer buf = new StringBuffer();
		for ( int i = 0; i < vecClm.size(); i++ ) {
			ColumnInfo col = (ColumnInfo )vecClm.get( i );
			buf.append( "\t\t\t\t+ \"" + col.getName() );
			if ( i < ( vecClm.size() - 1 )) {
				buf.append( ",\"" );
			} else {
				buf.append( "\"" );
			}
			buf.append( "\n" );
		}
		return buf.toString();
	}

	private String getWhere() {
		StringBuffer buf = new StringBuffer();
		int		nKey = 0;
		for ( int i = 0; i < vecClm.size(); i++ ) {
			ColumnInfo col = (ColumnInfo )vecClm.get( i );
			if ( !col.isKey()) {
				continue;
			}
			if ( nKey > 0 ) {
				buf.append( "and \"\n" );
			}
			nKey++;
			if ( col.getFetchType().equals( "String" ) ||
				col.getFetchType().equals( "Timestamp" ) ||
				col.getFetchType().equals( "Date" ))
			{
				buf.append( "\t\t\t+ \"" + col.getName() + " = '\" + " );
				buf.append( "getString( \"" + col.getName() + "\" ) + \"' " );
			} else {
				buf.append( "\t\t\t+ \"" + col.getName() + " = \" + " );
				buf.append( "getString( \"" + col.getName() + "\" ) + \" " );
			}
		}
		if ( nKey == 0 ) {
			buf.append( ";\n" );
		} else {
			buf.append( " \";\n" );
		}
		return buf.toString();
	}

	protected void printSelectByKeySql( PrintWriter out ) {
		out.println( "\t/**" );
		out.println( "\t* " + table
						+ "テーブルから、キーで指定されたレコードの" );
		out.println( "\t* 全カラムを取得するSQL文を実行します" );
		out.println( "\t* @return\tSQL文" );
		out.println( "\t*/" );
		out.println( "\tpublic String getSelectByKeySql() {" );
		if ( countKey() > 0 ) {
			out.println( "\t\tString where = \"where \"" );
			out.println( getWhere() );
		} else {
			out.println( "\t\tString where = \"\";" );
		}
		out.println( "\t\tString sql = getSelectSql( where );" );
		out.println( "\t\treturn sql;" );
		out.println( "\t}" );
		out.println( "" );
	}

	protected void printInsertSql( PrintWriter out ) {
		out.println( "\t/**" );
		out.println( "\t* " + table + "テーブルにオブジェクトの内容をinsertします" );
		out.println( "\t* @return\tSQL文" );
		out.println( "\t*/" );
		out.println( "\tpublic String getInsertSql() {" );
		out.println( "\t\tString  sql = \"insert into " + table + " (\"" );

		out.println( getAllColumn() );

		out.println( "\t\t+ \" ) values (\"" );
		out.print( "\t\t\t" + "+ \"" );
		for ( int i = 0; i < vecClm.size(); i++ ) {
			ColumnInfo col = (ColumnInfo )vecClm.get( i );
			String var;
			if ( isAudit( col.getName() )) {
				var = con.getSystemDate();
			} else {
				var = "?";
			}
			if ( i == vecClm.size() - 1 ) {
				out.println( var + ")\";" );
			} else {
				out.print( var + "," );
			}
		}
		out.println( "\t\treturn sql;" );
		out.println( "\t}" );
		out.println( "" );
	}

	private boolean isAudit( String name ) {
		for ( int i = 0; i < jparam.listAuditColumn().size(); i++ ) {
			String clm = (String )jparam.listAuditColumn().get( i );
			if ( clm.equals( name )) {
				return true;
			}
		}
		return false;
	}

	protected void printDeleteSql( PrintWriter out ) {
		out.println( "\t/**" );
		out.println( "\t* " + table + "テーブルから、キーで指定されたレコードを削除します" );
		out.println( "\t* @return\tSQL文" );
		out.println( "\t*/" );
		out.println( "\tpublic String getDeleteSql() {" );
		out.println( "\t\tString sql = \"delete from " + table + "\";" );
		if ( countKey() > 0 ) {
			out.println( "\t\tString where = \"where \"" );
			out.println( getWhere() );
			out.println( "\t\treturn sql + where;" );
		} else {
			out.println( "\t\treturn sql;" );
		}
		out.println( "\t}" );
		out.println( "" );
	}

	protected void printDebug( PrintWriter out ) {
		out.println( "\tpublic void printDebug() {" );
		for ( int i = 0; i < vecClm.size(); i++ ) {
			ColumnInfo col = (ColumnInfo )vecClm.get( i );
			out.println( "\t\tlogger.debug( this, \"" + col.getName()
							+ "=[\" + getString( \"" + col.getName()
							+ "\" ) + \"]\" );" );
		}
		out.println( "\t}" );
		out.println( "" );
	}

}
