package	jp.co.sds_corp.jdbrowser.status;

import java.awt.BorderLayout;
import java.awt.Point;
import java.awt.Window;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.text.DecimalFormat;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Properties;
import java.util.TreeMap;

import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.border.TitledBorder;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;

import jp.co.sds_corp.jdbrowser.SetupParam;
import jp.co.sds_corp.util.common.CmnProperty;
import jp.co.sds_corp.util.window.WindowUtility;

/**
* JVMの状況を表示します。
*/
public class JvmStatusPanel extends JPanel
{
	private static final long serialVersionUID = 1L;
	Window		parent;
	SetupParam	param;
	CmnProperty	prop;
	String		addHtml = "";
	JPanel		memPanel;
	JPanel		propPanel;
	JLabel		LBL_MEM = new JLabel();
	JTable		sheet;
	DefaultTableModel	model;
	PropertyDialog	propDlg;
	String		errTitle;
	String		errMsg;

	public JvmStatusPanel( Window parent, SetupParam param ) {
		this.parent = parent;
		this.param = param;
		prop = new CmnProperty( "resource.jdbrowser" );
		errTitle = prop.getString( "browser.err.title" );
		errMsg = prop.getString( "browser.err.message" );

		createControls();
	}

	protected void createControls() {
		setLayout( new BorderLayout() );

		propDlg = new PropertyDialog( parent, param );
		memPanel = new JPanel();
		memPanel.setLayout( new BoxLayout( memPanel, BoxLayout.Y_AXIS ));
		memPanel.add( LBL_MEM );
		add( memPanel, BorderLayout.NORTH );

		propPanel = new JPanel();
		String[] cols = { "key", "value" };
		model = new DefaultTableModel( cols, 0 ) {
			private static final long serialVersionUID = 1L;

			public boolean isCellEditable( int row, int col ) {
				return false;
			}
		};
		sheet = new JTable( model );
		sheet.addMouseListener( new MouseAdapter() {
			public void mouseClicked( MouseEvent ev ) {
				if ( ev.getClickCount() == 2 ) {
					showProperty( ev );
					return;
				}
			}
		} );

		TableColumn col;
		col = sheet.getColumnModel().getColumn( 1 );
		col.setCellRenderer( new ToolTipRenderer() );
		JScrollPane sp = new JScrollPane( sheet );
		sp.setBorder( new TitledBorder( "System property" ));
		propPanel.add( sp );
		add( sp, BorderLayout.CENTER );
	}

	protected void showProperty( MouseEvent ev ) {
		int row = sheet.rowAtPoint( new Point( ev.getX(), ev.getY() ));
		String key = (String )model.getValueAt( row, 0 );
		String val = (String )model.getValueAt( row, 1 );
		propDlg.setValue( key, val );

		WindowUtility.center( this, propDlg );
		propDlg.setVisible( true );
	}

	public JPanel getMemoryPanel() {
		return memPanel;
	}
	public JPanel getPropertyPanel() {
		return propPanel;
	}

	public void refresh() throws Exception {
		refreshMemory();
		refreshProperty();
	}

	protected void refreshMemory() throws Exception {
		Runtime rt = Runtime.getRuntime();
		DecimalFormat df = new DecimalFormat( "#,##0" );
		String text = "<html>"
			+ addHtml
			+ "<table>"
			+ "<tr><td>Max memory</td>"
				+ "<td align='right'>" + df.format( rt.maxMemory() ) + "</td></tr>"
			+ "<tr><td>Total memory</td>"
				+ "<td align='right'>" + df.format( rt.totalMemory() ) + "</td></tr>"
			+ "<tr><td>Free memory</td>"
				+ "<td align='right'>" + df.format( rt.freeMemory() ) + "</td></tr>"
			+ "</table></html>";
		LBL_MEM.setText( text );
	}

	protected void refreshProperty() throws Exception {
		model.setRowCount( 0 );
		TreeMap<String,String> map = new TreeMap<String,String>();
		Properties props = System.getProperties();
		Enumeration en = props.propertyNames();
		while ( en.hasMoreElements()) {
			String name = (String )en.nextElement();
			String value = props.getProperty( name );
			map.put( name, value );
		}
		Iterator itr = map.keySet().iterator();
		while( itr.hasNext()) {
			String name = (String )itr.next();
			String value = (String )map.get( name );
			addRow( name, value );
		}
	}

	private void addRow( String key, String value ) {
		Object[] row = new Object[2];
		row[0] = key;
		row[1] = value;
		model.addRow( row );
	}

	public void setAddHtml( String html ) {
		this.addHtml = html;
	}

}
