package	jp.co.sds_corp.jdbrowser.status;

import java.awt.Point;
import java.awt.Window;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.DriverPropertyInfo;

import javax.swing.JInternalFrame;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;

import jp.co.sds_corp.jdbrowser.SetupParam;
import jp.co.sds_corp.jdbrowser.connect.ConnectInfo;
import jp.co.sds_corp.util.common.CmnProperty;
import jp.co.sds_corp.util.db.DbAccess;
import jp.co.sds_corp.util.window.WindowUtility;

/**
* DB/システムの状況を表示します。
*/
public class StatusPanel extends JInternalFrame
{
	private static final long serialVersionUID = 1L;
	Window		parent;
	SetupParam	param;
	ConnectInfo	con;
	CmnProperty	prop;
	DbAccess	db;
	JTabbedPane	tab;
	JTable		sheet;
	DefaultTableModel	model;
	JvmStatusPanel		jvm;
	PropertyDialog	propDlg;
	String		errTitle;
	String		errMsg;

	public StatusPanel( Window parent, SetupParam param ) {
		super( "", true, true, true, true );
		this.parent = parent;
		this.param = param;
		prop = new CmnProperty( "resource.jdbrowser" );
		errTitle = prop.getString( "browser.err.title" );
		errMsg = prop.getString( "browser.err.message" );

		createControls();
	}

	protected void createControls() {
		setSize( 400, 500 );

		propDlg = new PropertyDialog( parent, param );
		tab = new JTabbedPane();
		String[] cols = { "key", "value" };
		model = new DefaultTableModel( cols, 0 ) {
			private static final long serialVersionUID = 1L;

			public boolean isCellEditable( int row, int col ) {
				return false;
			}
		};
		sheet = new JTable( model );
		sheet.addMouseListener( new MouseAdapter() {
			public void mouseClicked( MouseEvent ev ) {
				if ( ev.getClickCount() == 2 ) {
					showProperty( ev );
					return;
				}
			}
		} );

		TableColumn col;
		col = sheet.getColumnModel().getColumn( 1 );
		col.setCellRenderer( new ToolTipRenderer() );
		JScrollPane sp = new JScrollPane( sheet );
		tab.add( "DB", sp );

		jvm = new JvmStatusPanel( parent, param );
		tab.add( "JavaVM", jvm );
		getContentPane().add( tab );
	}

	protected void showProperty( MouseEvent ev ) {
		int row = sheet.rowAtPoint( new Point( ev.getX(), ev.getY() ));
		String key = (String )model.getValueAt( row, 0 );
		String val = (String )model.getValueAt( row, 1 );
		propDlg.setValue( key, val );

		WindowUtility.center( this, propDlg );
		propDlg.setVisible( true );
	}

	public void setDb( DbAccess db, ConnectInfo con ) {
		this.db = db;
		this.con = con;
		String title = prop.getString( "status.title" );
		setTitle( title + " - " + con.getDisplayName() );
	}

	public void setVisible( boolean var ) {
		if ( var == true ) {
			try {
				refresh();
			} catch ( Exception ex ) {
ex.printStackTrace();
				JOptionPane.showMessageDialog( this,
					errMsg + "\n" + ex, errTitle,
					JOptionPane.ERROR_MESSAGE );
			}
		}
		super.setVisible( var );
	}

	public void refresh() throws Exception {
		Connection con = db.getConnection();
		DatabaseMetaData meta = con.getMetaData();
		model.setRowCount( 0 );
		addRow( "DatabaseProductName", meta.getDatabaseProductName() );
		addRow( "DatabaseProductVersion", meta.getDatabaseProductVersion() );
		addRow( "DatabaseMajorVersion", String.valueOf( meta.getDatabaseMajorVersion() ));
		addRow( "DatabaseMinorVersion", String.valueOf( meta.getDatabaseMinorVersion() ));

		addRow( "DriverName", meta.getDriverName() );
		addRow( "DriverVersion", meta.getDriverVersion()  );
		addRow( "DriverMajorVersion", String.valueOf( meta.getDriverMajorVersion() ));
		addRow( "DriverMinorVersion", String.valueOf( meta.getDriverMinorVersion() ));
		addRow( "JDBCMajorVersion", String.valueOf( meta.getJDBCMajorVersion() ));
		addRow( "JDBCMinorVersion", String.valueOf( meta.getJDBCMinorVersion() ));
		addRow( "URL", meta.getURL() );
		addRow( "DriverClass", con.getClass().getName() );

		String url = meta.getURL();
		Driver drv = this.con.getSqlDriver();
		if ( drv == null ) {
			drv = DriverManager.getDriver( url );
		}
		if ( drv != null ) {
			DriverPropertyInfo[] infos = null;
			try {
				infos = drv.getPropertyInfo( url, null );
			} catch ( Exception ex ) {
			}
			if ( infos != null ) {
				for ( int i = 0; i < infos.length; i++ ) {
					addRow( infos[i].name, infos[i].value );
				}
			}
		}

		jvm.refresh();
	}

	protected void addRow( String key, String value ) {
		Object[] row = new Object[2];
		row[0] = key;
		row[1] = value;
		model.addRow( row );
	}

}
