package	jp.co.sds_corp.jdbrowser.table;

import java.sql.Array;
import java.sql.Blob;
import java.sql.CallableStatement;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.NClob;
import java.sql.PreparedStatement;
import java.sql.SQLClientInfoException;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.SQLXML;
import java.sql.Savepoint;
import java.sql.Statement;
import java.sql.Struct;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.Executor;

import jp.co.sds_corp.util.db.DbPreparedStatement;
import jp.co.sds_corp.util.log.CmnLogger;

public class ConnectionWrapper implements Connection
{
	static CmnLogger logger = CmnLogger.getInstance();
	boolean	debug;
	Connection con;

	public ConnectionWrapper( Connection con, boolean debug ) {
		this.con = con;
		this.debug = debug;
	}

	public void clearWarnings() throws SQLException {
		con.clearWarnings();
	}
	public void close() throws SQLException {
		if ( debug ) {
			// logger.sql( this, "close" );
			// logger.debug( this, "close", new Exception() );
		}
		// con.close();
	}
	public void commit() throws SQLException {
		if ( debug ) {
			logger.sql( this, "commit" );
		}
		con.commit();
	}
	public Statement createStatement() throws SQLException {
		return con.createStatement();
	}
	public Statement createStatement(int resultSetType, int resultSetConcurrency)
		throws SQLException
	{
		return con.createStatement(resultSetType, resultSetConcurrency);
	}
	public Statement createStatement(int resultSetType, int resultSetConcurrency, int resultSetHoldability)
		throws SQLException
	{
		return con.createStatement(resultSetType, resultSetConcurrency, resultSetHoldability);
	}
	public boolean getAutoCommit() throws SQLException {
		return con.getAutoCommit();
	}
	public String getCatalog() throws SQLException {
		return con.getCatalog();
	}
	public int getHoldability() throws SQLException {
		return con.getHoldability();
	}
	public DatabaseMetaData getMetaData() throws SQLException {
		return con.getMetaData();
	}
	public int getTransactionIsolation() throws SQLException {
		return con.getTransactionIsolation();
	}
	public Map<String,Class<?>> getTypeMap() throws SQLException {
		return con.getTypeMap();
	}
	public SQLWarning getWarnings() throws SQLException {
		return con.getWarnings();
	}
	public boolean isClosed() throws SQLException {
		return con.isClosed();
	}
	public boolean isReadOnly() throws SQLException {
		return con.isReadOnly();
	}
	public String nativeSQL(String sql) throws SQLException {
		return con.nativeSQL(sql);
	}
	public CallableStatement prepareCall(String sql) throws SQLException {
		if ( debug ) {
			logger.sql( this, "prepareCall sql[" + sql + "]" );
		}
		return con.prepareCall(sql);
	}
	public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency)
		throws SQLException
	{
		if ( debug ) {
			logger.sql( this, "prepareCall sql[" + sql + "]" );
		}
		return con.prepareCall(sql, resultSetType, resultSetConcurrency);
	}
	public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability)
		throws SQLException
	{
		if ( debug ) {
			logger.sql( this, "prepareCall sql[" + sql + "]" );
		}
		return con.prepareCall(sql, resultSetType, resultSetConcurrency, resultSetHoldability);
	}
	public PreparedStatement prepareStatement(String sql)
		throws SQLException
	{
		if ( debug ) {
			logger.sql( this, "prepare sql[" + sql + "]" );
		}
		return new DbPreparedStatement( con.prepareStatement(sql) );
	}
	public PreparedStatement prepareStatement(String sql, int autoGeneratedKeys)
		throws SQLException
	{
		if ( debug ) {
			logger.sql( this, "prepare sql[" + sql + "]" );
		}
		return new DbPreparedStatement( con.prepareStatement(sql, autoGeneratedKeys) );
	}
	public PreparedStatement prepareStatement(String sql, int[] columnIndexes)
		throws SQLException
	{
		if ( debug ) {
			logger.sql( this, "prepare sql[" + sql + "]" );
		}
		return new DbPreparedStatement( con.prepareStatement(sql, columnIndexes) );
	}
	public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency)
		throws SQLException
	{
		if ( debug ) {
			logger.sql( this, "prepare sql[" + sql + "]" );
		}
		return new DbPreparedStatement( con.prepareStatement(sql, resultSetType, resultSetConcurrency) );
	}
	public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability)
		throws SQLException
	{
		if ( debug ) {
			logger.sql( this, "prepare sql[" + sql + "]" );
		}
		return new DbPreparedStatement( con.prepareStatement(sql, resultSetType, resultSetConcurrency, resultSetHoldability) );
	}
	public PreparedStatement prepareStatement(String sql, String[] columnNames)
		throws SQLException
	{
		if ( debug ) {
			logger.sql( this, "prepare sql[" + sql + "]" );
		}
		return new DbPreparedStatement( con.prepareStatement(sql, columnNames) );
	}
	public void releaseSavepoint(Savepoint savepoint) throws SQLException {
		con.releaseSavepoint(savepoint);
	}
	public void rollback() throws SQLException {
		if ( debug ) {
			logger.sql( this, "rollback" );
		}
		con.rollback();
	}
	public void rollback(Savepoint savepoint) throws SQLException {
		if ( debug ) {
			logger.sql( this, "rollback:" + savepoint);
		}
		con.rollback(savepoint);
	}
	public void setAutoCommit(boolean autoCommit) throws SQLException {
		con.setAutoCommit(autoCommit);
	}
	public void setCatalog(String catalog) throws SQLException {
		con.setCatalog(catalog);
	}
	public void setHoldability(int holdability) throws SQLException {
		con.setHoldability(holdability);
	}
	public void setReadOnly(boolean readOnly) throws SQLException {
		con.setReadOnly(readOnly);
	}
	public Savepoint setSavepoint() throws SQLException {
		return con.setSavepoint();
	}
	public Savepoint setSavepoint(String name) throws SQLException {
		return con.setSavepoint(name);
	}
	public void setTransactionIsolation(int level) throws SQLException {
		con.setTransactionIsolation(level);
	}
	public void setTypeMap(Map<String,Class<?>> map) throws SQLException {
		con.setTypeMap(map) ;
	}

	public Array createArrayOf(String typeName, Object[] elements)
		throws SQLException
	{
		return con.createArrayOf(typeName, elements);
	}
	public Blob createBlob() throws SQLException {
		return con.createBlob();
	}
	public Clob createClob() throws SQLException {
		return con.createClob();
	}
	public NClob createNClob() throws SQLException {
		return con.createNClob();
	}
	public SQLXML createSQLXML() throws SQLException {
		return con.createSQLXML();
	}
	public Struct createStruct(String typeName, Object[] attributes)
		throws SQLException
	{
		return con.createStruct(typeName, attributes);
	}

	public Properties getClientInfo() throws SQLException {
		return con.getClientInfo();
	}

	public String getClientInfo(String name) throws SQLException {
		return con.getClientInfo(name);
	}

	public void setClientInfo(Properties properties)
		throws SQLClientInfoException
	{
		con.setClientInfo(properties);
	}

	public void setClientInfo(String name, String value)
		throws SQLClientInfoException
	{
		con.setClientInfo(name, value);
	}

	public boolean isValid(int timeout) throws SQLException {
		return con.isValid(timeout);
	}

	public boolean isWrapperFor(java.lang.Class<?> iface) throws SQLException {
		return con.isWrapperFor(iface);
	}
	public <T> T unwrap(Class<T> iface) throws SQLException {
		return con.unwrap( iface );
	}

	//-------------------------- Java7 -----------------------------
	@Override
	public int getNetworkTimeout() throws SQLException {
		return con.getNetworkTimeout();
	}

	@Override
	public void setNetworkTimeout(Executor executor, int milliseconds)
		throws SQLException
	{
		con.setNetworkTimeout(executor, milliseconds);
	}

	@Override
	public void abort(Executor executor) throws SQLException {
		con.abort(executor);
	}

	@Override
	public String getSchema() throws SQLException {
		return con.getSchema();
	}

	@Override
	public void setSchema(String schema) throws SQLException {
		con.setSchema(schema);
	}

}
