package	jp.co.sds_corp.jdbrowser.table;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.ResultSet;
import java.util.Vector;

import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JToolBar;
import javax.swing.ListCellRenderer;
import javax.swing.ListSelectionModel;
import javax.swing.UIManager;
import javax.swing.table.JTableHeader;

import jp.co.sds_corp.jdbrowser.SetupParam;
import jp.co.sds_corp.jdbrowser.connect.ConnectInfo;
import jp.co.sds_corp.util.common.CmnProperty;
import jp.co.sds_corp.util.db.DbAccess;
import jp.co.sds_corp.util.db.DbTable;
import jp.co.sds_corp.util.window.WindowUtility;

/**
* SQLの結果を表示します。
*/
public class QueryViewer extends JInternalFrame
{
	private static final long serialVersionUID = 1L;
	Window		parent;
	SetupParam	param;
	CmnProperty	prop;
	DbAccess	db;
	ConnectInfo	con;
	String		catalog;
	String		schema;
	String		table;
	JScrollPane sp;
	RowHeader	rowHd;
	ViewerSheet	sheet;
	ViewerSheet.Model	model;
	JToolBar	toolBar;
	JButton		BTN_REFRESH;
	JButton		BTN_EXEC;
	JButton		BTN_SAVE;
	Vector<String>	vecHist = new Vector<String>();
	JComboBox	CMB_HIST = new JComboBox( vecHist );
	JTextArea	TXT_SQL = new JTextArea();
	String		errTitle;
	String		errMsg;

	public QueryViewer( Window parent, SetupParam param, DbAccess db,
			ConnectInfo con )
	{
		super( "", true, true, true, true );
		this.parent = parent;
		this.param = param;
		this.db = db;
		this.con = con;
		prop = new CmnProperty( "resource.jdbrowser" );
		errTitle = prop.getString( "query.err.title" );
		errMsg = prop.getString( "query.err.message" );
		createControls();
	}

	protected void createControls() {
		String title = prop.getString( "query.title" );
		setTitle( title + " - " + " " + con.getDisplayName() );
		setSize( 650, 300 );

		JSplitPane panel1 = new JSplitPane( JSplitPane.VERTICAL_SPLIT, true );
		JPanel panel2 = new JPanel();
		panel2.setLayout( new BorderLayout() );
		CMB_HIST.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				String text = (String )CMB_HIST.getSelectedItem();
				TXT_SQL.setText( text );
			}
		});
		panel2.add( CMB_HIST, BorderLayout.NORTH );
		String text;
		TXT_SQL.setRows( 20 );
		text = prop.getString( "query.sql.tooltip" );
		TXT_SQL.setToolTipText( text );
		sp = new JScrollPane( TXT_SQL );
		panel2.add( sp, BorderLayout.CENTER );
		panel1.add( panel2 );

		JPanel panel3 = new JPanel( new BorderLayout() );
		toolBar = createToolBar();
		panel3.add( toolBar, BorderLayout.NORTH );

		sheet = new ViewerSheet();
		sheet.setEditable( false );
		model = (ViewerSheet.Model )sheet.getModel();

		sp = setTable();
		panel3.add( sp, BorderLayout.CENTER );
		panel1.add( panel3 );
		panel1.setDividerLocation( 100 );
		getContentPane().add( panel1 );
	}

	protected JToolBar createToolBar() {
		JToolBar toolBar = new JToolBar( "toolbar" );
		String text;
		BTN_REFRESH = new JButton();
		text = prop.getString( "query.refresh.tooltip" );
		BTN_REFRESH.setToolTipText( text );
		WindowUtility.setButtonIcon( this, BTN_REFRESH,
			"/resource/exec.png", "/resource/exec_d.png" );
		BTN_REFRESH.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				try {
					execute();
				} catch ( Exception ex ) {
					error( ex, true );
				}
			}
		} );
		toolBar.add( BTN_REFRESH );

/*
		BTN_SAVE = new JButton();
		text = prop.getString( "query.save.tooltip" );
		BTN_SAVE.setToolTipText( text );
		WindowUtility.setButtonIcon( this, BTN_SAVE, "/resource/save.png" );
		BTN_SAVE.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				try {
					save();
				} catch ( Exception ex ) {
					error( ex, true );
				}
			}
		} );
		BTN_SAVE.setEnabled( false );
		toolBar.add( BTN_SAVE );
*/
		return toolBar;
	}

	private void error( Exception ex, boolean rollback ) {
		if ( rollback ) {
			try {
				db.rollback();
			} catch ( Exception ex2 ) {
			}
		}
ex.printStackTrace();
		JOptionPane.showMessageDialog( this,
			errMsg + "\n" + ex, errTitle,
			JOptionPane.ERROR_MESSAGE );
	}

	private JScrollPane setTable() {
		JScrollPane sp = new JScrollPane( sheet );
		rowHd = new RowHeader( sheet );
		rowHd.setSelectionMode( ListSelectionModel.MULTIPLE_INTERVAL_SELECTION );
		sp.setRowHeaderView( rowHd );
		sp.getRowHeader().setPreferredSize(
			new Dimension( 30, sheet.getPreferredSize().height ));
		sp.setHorizontalScrollBarPolicy( JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS );
		return sp;
	}

	class RowHeader extends JList
	{
		private static final long serialVersionUID = 1L;
		JTableHeader header;

		public RowHeader( JTable table ) {
			super( new DefaultListModel() );
			header = table.getTableHeader();
			setFixedCellHeight( table.getRowHeight());
			setCellRenderer( new RowHeaderRenderer( table.getTableHeader()));
		}
		public Color getBackground() {
			if ( header == null ) {
				return super.getBackground();
			}
			return header.getBackground();
		}
	}
	class RowHeaderRenderer extends JLabel implements ListCellRenderer
	{
		private static final long serialVersionUID = 1L;
		private JTableHeader header;

		public RowHeaderRenderer( JTableHeader header ) {
			this.header = header;
		}
		public Component getListCellRendererComponent( JList list, Object value,
			int index, boolean isSelected, boolean cellHasFocus )
		{
			setOpaque( true );
			setBackground( header.getBackground() );
			setBorder( UIManager.getBorder( "TableHeader.cellBorder" ));
			setHorizontalAlignment( RIGHT );
			setFont( header.getFont() );
			setText(( value == null ) ? "" : value.toString());
			return this;
		}
	}

	public void setVisible( boolean var ) {
		super.setVisible( var );
		if ( var ) {
			TXT_SQL.requestFocus();
		}
	}

	public void execute()
		throws Exception
	{
		sheet.setInit( false );
		model.setRowCount( 0 );
		((DefaultListModel )rowHd.getModel()).clear();
		String sql = TXT_SQL.getText();
		if ( sql == null || sql.length() == 0 ) {
			return;
		}
		DbTable tbl = new DbTable();
		boolean ret = db.execute( sql );
		if ( !ret ) {
			int nUpd = db.getUpdateCount();
			db.commit();
			addSql( sql );
			String msg = "complete.";
			if ( nUpd > 0 ) {
				msg = nUpd + " rows updated.";
			}
			JOptionPane.showMessageDialog( this, msg, "Result",
				JOptionPane.INFORMATION_MESSAGE );
			return;
		}
		ResultSet rset = db.getResultSet();
		int nRow = 0;
		for ( ;; ) {
			if ( !db.fetch( tbl )) {
				break;
			}
			nRow++;
			sheet.addRow( tbl );
			((DefaultListModel )rowHd.getModel()).addElement( "" + nRow );
		}
		if ( nRow == 0 ) {
			sheet.initColumn( tbl );
		}

		addSql( sql );
	}

	private void addSql( String sql ) {
		int idx = vecHist.indexOf( sql );
		if ( idx < 0 ) {
			DefaultComboBoxModel model = (DefaultComboBoxModel )CMB_HIST.getModel();
			model.addElement( sql );
		} else {
			CMB_HIST.setSelectedIndex( idx );
		}
	}

	protected void save() throws Exception {
	}

}
