package	jp.co.sds_corp.jdbrowser.table;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.Types;
import java.text.SimpleDateFormat;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableCellEditor;

import jp.co.sds_corp.jdbrowser.ColumnInfo;
import jp.co.sds_corp.jdbrowser.SetupParam;
import jp.co.sds_corp.jdbrowser.connect.ConnectInfo;
import jp.co.sds_corp.jdbrowser.event.TableEvent;
import jp.co.sds_corp.jdbrowser.event.TableEventListener;
import jp.co.sds_corp.jdbrowser.generator.GeneratorFactory;
import jp.co.sds_corp.jdbrowser.generator.IGenerator;
import jp.co.sds_corp.jdbrowser.util.BrowserUtil;
import jp.co.sds_corp.util.common.CmnProperty;
import jp.co.sds_corp.util.db.DbAccess;
import jp.co.sds_corp.util.db.DbException;
import jp.co.sds_corp.util.db.DbTable;
import jp.co.sds_corp.util.window.WindowUtility;

/**
* tableの情報を表示します。
*/
public class TableDesign extends JInternalFrame
{
	private static final long serialVersionUID = 1L;
	Window		parent;
	SetupParam	param;
	CmnProperty	prop;
	DbAccess	db;
	ConnectInfo	con;
	String		catalog;
	String		schema;
	String		table;
	boolean		changed;
	Vector<TableEventListener>		vecLsn = new Vector<TableEventListener>();
	Vector<ColumnInfo>		vecCol = new Vector<ColumnInfo>();
	Vector<String>		vecKey = new Vector<String>();
	TableDesignSheet	sheet;
	TableDesignSheet.Model	model;
	JTextField	TXT_SCH = new JTextField();
	JTextField	TXT_CAT = new JTextField();
	JTextField	TXT_TBL = new JTextField();
	JToolBar	toolBar;
	JButton		BTN_EXEC;
	JButton		BTN_ADD;
	JButton		BTN_DEL;
	JButton		BTN_DOC;
	JButton		BTN_JAVA;
	String		errTitle;
	String		errMsg;
	int			mode = MODE_UPD;
	public static int MODE_NEW = 0;
	public static int MODE_UPD = 1;

	public TableDesign( Window parent, SetupParam param ) {
		super( "", true, true, true, true );
		this.parent = parent;
		this.param = param;
		prop = new CmnProperty( "resource.jdbrowser" );
		errTitle = prop.getString( "design.err.title" );
		errMsg = prop.getString( "design.err.message" );

		createControls();
	}

	protected void createControls() {
		setSize( 650, 300 );

		Box panel1 = new Box( BoxLayout.Y_AXIS );
		Box panel2 = new Box( BoxLayout.X_AXIS );
		panel2.setMaximumSize( new Dimension( 1000, 10 ));
		panel2.add( new JLabel( "Catalog:" ));
		TXT_CAT.setEditable( false );
		panel2.add( TXT_CAT );
		panel2.add( new JLabel( "Schema:" ));
		TXT_SCH.setEditable( false );
		panel2.add( TXT_SCH );
		panel2.add( new JLabel( "Table:" ));
		TXT_TBL.setEditable( false );
		panel2.add( TXT_TBL );
		panel1.add( panel2 );

		JPanel	panel3 = new JPanel( new BorderLayout() );
		toolBar = createToolBar();
		panel3.add( toolBar, BorderLayout.NORTH );

		sheet = new TableDesignSheet();
		model = (TableDesignSheet.Model )sheet.getModel();
		model.addTableModelListener( new TableModelListener() {
			public void tableChanged( TableModelEvent ev ) {
				setChanged( true );
			}
		} );
		JScrollPane sp = new JScrollPane( sheet );
		panel3.add( sp, BorderLayout.CENTER );
		panel1.add( panel3 );
		getContentPane().add( panel1 );
	}

	protected JToolBar createToolBar() {
		JToolBar toolBar = new JToolBar( "toolbar" );
		String text;
		BTN_EXEC = new JButton();
		WindowUtility.setButtonIcon( this, BTN_EXEC,
			"/resource/exec.png", "/resource/exec_d.png" );
		BTN_EXEC.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				try {
					if ( mode == MODE_NEW ) {
						createTable();
					} else {
						updateTable();
					}
				} catch ( Exception ex ) {
					fireTableChanged( table );
					error( ex, true );
				}
			}
		} );
		text = prop.getString( "design.exec.tooltip" );
		BTN_EXEC.setToolTipText( text );
		toolBar.add( BTN_EXEC );
		toolBar.addSeparator();

		BTN_ADD = new JButton();
		WindowUtility.setButtonIcon( this, BTN_ADD, "/resource/plus.png" );
		BTN_ADD.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				addRow();
			}
		} );
		text = prop.getString( "design.add.tooltip" );
		BTN_ADD.setToolTipText( text );
		toolBar.add( BTN_ADD );
		BTN_DEL = new JButton();
		WindowUtility.setButtonIcon( this, BTN_DEL, "/resource/minus.png" );
		BTN_DEL.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				delRow();
			}
		} );
		text = prop.getString( "design.del.tooltip" );
		BTN_DEL.setToolTipText( text );
		toolBar.add( BTN_DEL );
		toolBar.addSeparator();

		BTN_DOC = new JButton();
		WindowUtility.setButtonIcon( this, BTN_DOC, "/resource/doc.png" );
		BTN_DOC.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				generateDocument();
			}
		} );
		text = prop.getString( "design.doc.tooltip" );
		BTN_DOC.setToolTipText( text );
		toolBar.add( BTN_DOC );
		BTN_JAVA = new JButton();
		WindowUtility.setButtonIcon( this, BTN_JAVA, "/resource/java.png" );
		BTN_JAVA.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				generateJava();
			}
		} );
		text = prop.getString( "design.java.tooltip" );
		BTN_JAVA.setToolTipText( text );
		toolBar.add( BTN_JAVA );
		return toolBar;
	}

	private void error( Exception ex, boolean rollback ) {
		if ( rollback ) {
			try {
				db.rollback();
			} catch ( Exception ex2 ) {
			}
		}
ex.printStackTrace();
		JOptionPane.showMessageDialog( this,
			errMsg + "\n" + ex, errTitle,
			JOptionPane.ERROR_MESSAGE );
	}

	protected void generateDocument() {
		try {
			IGenerator gen = GeneratorFactory.createDocumentGenerator( param );
			gen.select( db, con, catalog, schema, table );
			gen.generate( parent );
		} catch ( Exception ex ) {
			error( ex, false );
		}
	}

	protected void generateJava() {
		try {
			IGenerator gen = GeneratorFactory.createJavaGenerator( param );
			gen.select( db, con, catalog, schema, table );
			gen.generate( parent );
		} catch ( Exception ex ) {
			error( ex, false );
		}
	}

	public void addTableEventListener( TableEventListener lsn ) {
		vecLsn.add( lsn );
	}
	public void removeTableEventListener( TableEventListener lsn ) {
		vecLsn.remove( lsn );
	}
	public void fireTableChanged( String table ) {
		TableEvent ev = new TableEvent( this, catalog, schema, table );
		for ( int i = 0; i < vecLsn.size(); i++ ) {
			TableEventListener lsn = (TableEventListener )vecLsn.get( i );
			lsn.tableChanged( ev );
		}
	}

	public void newTable( DbAccess db,
			ConnectInfo con, String catalog, String schema )
		throws Exception
	{
		this.db = db;
		this.con = con;
		this.catalog = catalog;
		this.schema = schema;
		this.mode = MODE_NEW;

		selectType( db, catalog, schema );

		TXT_CAT.setText( catalog );
		TXT_SCH.setText( schema );
		TXT_TBL.setEditable( true );

		String title = prop.getString( "design.title" );
		String name = BrowserUtil.getName( catalog, schema, "" );
		setTitle( title + " - " + con.getDisplayName() + " " + name );
		setChanged( false );
	}

	public void select( DbAccess db,
			ConnectInfo con, String catalog, String schema, String table )
		throws Exception
	{
		this.db = db;
		this.con = con;
		this.catalog = catalog;
		this.schema = schema;
		this.table = table;
		this.mode = MODE_UPD;

		selectKey( db, catalog, schema, table );
		selectTable( db, catalog, schema, table );
		selectComment( db, catalog, schema, table );
//		selectIndex( db, catalog, schema, table );
		selectType( db, catalog, schema );

		model.clearRow();
		for ( int row = 0; row < vecCol.size(); row++ ) {
			ColumnInfo info = (ColumnInfo )vecCol.get( row );
			model.addRow( info );
		}

		TXT_CAT.setText( catalog );
		TXT_SCH.setText( schema );
		TXT_TBL.setText( table );

		String title = prop.getString( "design.title" );
		String name = BrowserUtil.getName( catalog, schema, table );
		setTitle( title + " - " + con.getDisplayName() + " " + name );
		setChanged( false );
	}

	public void selectTable( DbAccess db,
			String catalog, String schema, String table )
		throws Exception
	{
		vecCol.clear();
		DatabaseMetaData meta = db.getConnection().getMetaData();
		ResultSet rset = meta.getColumns( catalog, schema, table, null );
		for ( ;; ) {
			if ( !rset.next()) {
				break;
			}
			ColumnInfo info = new ColumnInfo( rset );
			if ( vecKey.indexOf( info.getName() ) >= 0 ) {
				info.setKey( true );
			}
			vecCol.add( info );
		}
	}

	public void selectComment( DbAccess db,
			String catalog, String schema, String table )
		throws Exception
	{
		String sql = con.getSelectComment();
		if ( sql == null ) {
			return;
		}
		sql = replaceVar( sql, "$CATALOG", catalog );
		sql = replaceVar( sql, "$SCHEMA", schema );
		sql = replaceVar( sql, "$TABLE", table );
		for ( int row = 0; row < vecCol.size(); row++ ) {
			ColumnInfo col = (ColumnInfo )vecCol.get( row );
			if ( col.getRemarks() != null && col.getRemarks().length() > 0 ) {
				continue;
			}
			String exec;
			exec = replaceVar( sql, "$COLUMN", col.getName() );
			db.executeQuery( exec );
			DbTable tbl = new DbTable();
			if ( !db.fetch( tbl )) {
				throw( new DbException( "column not found" ));
			}
			db.cancel();
			col.setRemarks( tbl.getString( 0 ));
		}
	}

	public void selectKey( DbAccess db,
			String catalog, String schema, String table )
		throws Exception
	{
		vecKey.clear();
		DatabaseMetaData meta = db.getConnection().getMetaData();
		ResultSet rset = meta.getPrimaryKeys( catalog, schema, table );
		for ( ;; ) {
			if ( !rset.next()) {
				break;
			}
			vecKey.add( rset.getString( "COLUMN_NAME" ));
		}
	}

	public void selectIndex( DbAccess db,
			String catalog, String schema, String table )
		throws Exception
	{
		DatabaseMetaData meta = db.getConnection().getMetaData();
		ResultSet rset = meta.getIndexInfo( catalog, schema, table, false, false );
		for ( ;; ) {
			if ( !rset.next()) {
				break;
			}
System.out.println( rset.getString( "INDEX_NAME" ) + ":"
	+ rset.getString( "COLUMN_NAME" ) + ":"
	+ rset.getString( "NON_UNIQUE" ) + ":"
	+ rset.getString( "ASC_OR_DESC" ) );
		}
	}

	public void selectType( DbAccess db, String catalog, String schema )
		throws Exception
	{
		Vector<String> vec = new Vector<String>();
		DatabaseMetaData meta = db.getConnection().getMetaData();
		ResultSet rset = meta.getTypeInfo();
		for ( ;; ) {
			if ( !rset.next()) {
				break;
			}
			int jType = rset.getInt( "DATA_TYPE" );
			if ( jType == Types.OTHER || jType >= 2000 ) {
				continue;
			}
			String type = rset.getString( "TYPE_NAME" );
			if ( vec.indexOf( type ) < 0 ) {
				vec.add( type );
			}
		}
		sheet.setTypeList( vec );
	}

	protected void setChanged( boolean var ) {
		BTN_EXEC.setEnabled( var );
		this.changed = var;
	}

	protected void addRow() {
		int nRow = sheet.getSelectedRow();
		if ( nRow < 0 ) {
			model.addRow();
		} else {
			stopEditing( nRow );
			model.insertRow( nRow );
		}
	}
	protected void delRow() {
		int nRow = sheet.getSelectedRow();
		if ( nRow >= 0 ) {
			stopEditing( nRow );
			model.delRow( nRow );
		}
	}

	protected void stopEditing( int row ) {
		for ( int i = 0; i < model.getColumnCount(); i++ ) {
			TableCellEditor editor = sheet.getCellEditor( row, i );
			if ( editor != null ) {
				try {
					editor.stopCellEditing();
				} catch ( NullPointerException ne ) {
				}
			}
		}
	}

	protected void createTable() throws Exception {
		int nRow = sheet.getSelectedRow();
		stopEditing( nRow );

		table = TXT_TBL.getText();
		if ( table == null || table.length() == 0 ) {
			String title = prop.getString( "design.tblerr.title" );
			String msg = prop.getString( "design.tblerr.message" );
			JOptionPane.showMessageDialog( this,
				msg, title,
				JOptionPane.ERROR_MESSAGE );
			return;
		}
		createNewTable();
		db.commit();
		fireTableChanged( table );

		select( db, con, catalog, schema, table );
		BTN_EXEC.setEnabled( false );
		mode = MODE_UPD;
	}

	protected void updateTable() throws Exception {
		int nRow = sheet.getSelectedRow();
		stopEditing( nRow );

		SimpleDateFormat sdf = new SimpleDateFormat( "yyyyMMdd_HHmmss" );
		String tmpTbl = table + "_" + sdf.format( new java.util.Date() );

		renameOriginal( tmpTbl );
		createNewTable();
		db.commit();
		fireTableChanged( table );
		try {
			copyFromTemp( tmpTbl );
		} catch ( Exception ex ) {
			String title = prop.getString( "design.inserr.title" );
			String msg = prop.getString( "design.inserr.message" );
			JOptionPane.showMessageDialog( this,
				msg + " [" + tmpTbl + "]\n" + ex, title,
				JOptionPane.ERROR_MESSAGE );
		}
		db.commit();
		try {
			dropOriginal( tmpTbl );
		} catch ( Exception ex ) {
			String title = prop.getString( "design.droperr.title" );
			String msg = prop.getString( "design.droperr.message" );
			JOptionPane.showMessageDialog( this,
				msg + " [" + tmpTbl + "]\n" + ex, title,
				JOptionPane.ERROR_MESSAGE );
		}
		db.commit();
		fireTableChanged( table );

		selectKey( db, catalog, schema, table );
		selectTable( db, catalog, schema, table );
		BTN_EXEC.setEnabled( false );
	}

	protected void renameOriginal( String tmpTbl ) throws Exception {
		String sql;
		sql = "ALTER TABLE " + table + " RENAME TO " + tmpTbl;
		db.execute( sql );
	}

	protected void createNewTable() throws Exception {
		String sql;
		vecCol.clear();
		int nKey = 0;
		sql = "CREATE TABLE " + table + "(";
		for ( int row = 0; row < model.getRowCount(); row++ ) {
			if ( row > 0 ) {
				sql += ",";
			}
			String name = (String )model.getValueAt( row, 0 );
			String type = (String )model.getValueAt( row, 1 );
if ( type.equals( "bpchar" )) {
	type = "char";
}
			String prec = (String )model.getValueAt( row, 2 );
			String scale = (String )model.getValueAt( row, 3 );
			Boolean key = (Boolean )model.getValueAt( row, 4 );
			Boolean nullable = (Boolean )model.getValueAt( row, 5 );
			String defs = (String )model.getValueAt( row, 6 );
			String comment = (String )model.getValueAt( row, 7 );
			ColumnInfo info = new ColumnInfo( name, type, prec, scale,
				key, nullable, defs, comment );
			vecCol.add( info );
			sql += info.getName() + " " + info.getType();
			if ( info.needPrecision()) {
				sql += "(" + info.getPrecision();
				if ( info.needScale()) {
					sql += "," + info.getScale();
				}
				sql += ")";
			}
			if ( defs != null && defs.length() > 0 ) {
				sql += " DEFAULT " + defs;
			}
			if ( !info.isNullable() ) {
				sql += " not null";
			}
			String cmtSql = con.getUpdateComment();
			if ( con.getUpdateType() == ConnectInfo.COMMENT_CREATE &&
				cmtSql != null && comment != null && comment.length() > 0 )
			{
				cmtSql = replaceVar( cmtSql, "$CATALOG", catalog );
				cmtSql = replaceVar( cmtSql, "$SCHEMA", schema );
				cmtSql = replaceVar( cmtSql, "$TABLE", table );
				cmtSql = replaceVar( cmtSql, "$COLUMN", info.getName() );
				cmtSql = replaceVar( cmtSql, "$COMMENT", info.getRemarks() );
				sql += " " + cmtSql;
			}
			if ( info.isKey() ) {
				nKey++;
			}
		}
		if ( nKey > 0 ) {
			sql += ", PRIMARY KEY (";
			int nPos = 0;
			for ( int row = 0; row < vecCol.size(); row++ ) {
				ColumnInfo info = (ColumnInfo )vecCol.get( row );
				if ( !info.isKey()) {
					continue;
				}
				if ( nPos > 0 ) {
					sql += ",";
				}
				sql += info.getName();
				nPos++;
			}
			sql += ")";
		}
		sql += ")";
		db.execute( sql );

		createComment();
	}

	private String replaceVar( String src, String from, String to ) {
		String ret = src;
		for ( ;; ) {
			int idx = ret.indexOf( from );
			if ( idx < 0 ) {
				break;
			}
			ret = ret.substring( 0, idx ) + to
					+ ret.substring( idx + from.length() );
		}
		return ret;
	}

	protected void createComment() throws Exception {
		String sql = con.getUpdateComment();
		if ( con.getUpdateType() != ConnectInfo.COMMENT_UPDATE || sql == null ) {
			return;
		}
		sql = replaceVar( sql, "$CATALOG", catalog );
		sql = replaceVar( sql, "$SCHEMA", schema );
		sql = replaceVar( sql, "$TABLE", table );
		for ( int row = 0; row < vecCol.size(); row++ ) {
			ColumnInfo col = (ColumnInfo )vecCol.get( row );
			if ( col.getRemarks() == null || col.getRemarks().length() == 0 ) {
				continue;
			}
			String exec;
			exec = replaceVar( sql, "$COLUMN", col.getName() );
			exec = replaceVar( exec, "$COMMENT", col.getRemarks() );
			db.execute( exec );
		}
	}

	protected void copyFromTemp( String tmpTbl ) throws Exception {
		String sql;
		sql = "INSERT INTO " + table + "(";
		int nPos = 0;
		for ( int row = 0; row < vecCol.size(); row++ ) {
			ColumnInfo org = model.getRow( row );
			ColumnInfo col = (ColumnInfo )vecCol.get( row );
			if ( org.getName() == null ) {
				continue;
			}
			if ( nPos > 0 ) {
				sql += ",";
			}
			sql += col.getName();
			nPos++;
		}
		sql += ") SELECT ";
		nPos = 0;
		for ( int row = 0; row < vecCol.size(); row++ ) {
			ColumnInfo org = model.getRow( row );
			ColumnInfo col = (ColumnInfo )vecCol.get( row );
			if ( org.getName() == null ) {
				continue;
			}
			if ( nPos > 0 ) {
				sql += ",";
			}
			sql += org.getName();
			nPos++;
		}
		sql += " FROM " + tmpTbl;
		db.execute( sql );
	}

	protected void dropOriginal( String tmpTbl ) throws Exception {
		String sql;
		sql = "DROP TABLE " + tmpTbl; // + " CASCADE";
		db.execute( sql );
	}

}
