package	jp.co.sds_corp.jdbrowser.table;

import java.awt.Component;
import java.sql.Types;
import java.util.Vector;

import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;

import jp.co.sds_corp.jdbrowser.ColumnInfo;
import jp.co.sds_corp.util.common.CmnProperty;
import jp.co.sds_corp.util.db.DbColumn;
import jp.co.sds_corp.util.db.DbTable;

/**
* table/viewのデータを表示します。
*/
public class ViewerSheet extends JTable
{
	private static final long serialVersionUID = 1L;
	CmnProperty	prop;
	Model model;
	boolean	init = false;
	int[]	widths;
	boolean	editable = true;
	Vector<ColumnInfo>	vecCol = new Vector<ColumnInfo>();
	boolean[]	changed;

	public ViewerSheet() {
		prop = new CmnProperty( "resource.jdbrowser" );
		setAutoResizeMode( AUTO_RESIZE_OFF );
		// setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		model = new Model();
		model.addTableModelListener( new TableModelListener() {
			public void tableChanged( TableModelEvent ev ) {
				if ( ev.getColumn() >= 0 ) {
					changed[ev.getColumn()] = true;
				}
			}
		});
		setModel( model ); 
	}

	public void setInit( boolean var ) {
		this.init = var;
	}
	public void addRow( DbTable tbl ) {
		if ( !init ) {
			initColumn( tbl );
			init = true;
		}
		Vector<String> vec = new Vector<String>();
		for ( int i = 0; i < tbl.getColumnCount(); i++ ) {
			vec.add( tbl.getString( i ));
			changed[i] = false;
		}
		model.addRow( vec );
	}

	public void setEditable( boolean var ) {
		this.editable = var;
	}

	public boolean isCellEditable( int row, int column ) {
		return this.editable;
	}

	public ColumnInfo getColumn( int col ) {
		ColumnInfo clm = (ColumnInfo ) vecCol.get( col );
		return clm;
	}

	public boolean isChanged() {
		for ( int i = 0; i < changed.length; i++ ) {
			if ( changed[i] ) {
				return true;
			}
		}
		return false;
	}
	public void clearChanged() {
		for ( int i = 0; i < changed.length; i++ ) {
			changed[i] = false;
		}
	}
	public boolean isColumnChanged( int col ) {
		return changed[col];
	}

	protected void initColumn( DbTable tbl ) {
		model.setColumnCount( 0 );
		changed = new boolean[tbl.getColumnCount()];
		vecCol.clear();
		NumericCellRenderer numRen = new NumericCellRenderer();
		for ( int i = 0; i < tbl.getColumnCount(); i++ ) {
			DbColumn col = tbl.getColumn( i );
			model.addColumn( col.getName() );
			ColumnInfo info = new ColumnInfo( col );
			vecCol.add( info );
		}

		for ( int i = 0; i < tbl.getColumnCount(); i++ ) {
			DbColumn col = tbl.getColumn( i );
			TableColumn tcol = getColumnModel().getColumn( i );
			int size = col.getSize();
			switch ( col.getType() ) {
			case Types.DATE:
			case Types.TIMESTAMP:
				size = 21;
				break;
			case Types.TIME:
				size = 8;
				break; 
			case Types.BIT:
			case Types.BOOLEAN:
				size = 5;
				break;
			case Types.TINYINT:
			case Types.BIGINT:
			case Types.INTEGER:
			case Types.SMALLINT:
			case Types.NUMERIC:
			case Types.DECIMAL:
				size = col.getPrecision() + col.getScale() + 1;
				tcol.setCellRenderer( numRen );
				break;
			}
			int width = 0;
			if ( size > col.getName().length() ) {
				width = size;
			} else {
				width = col.getName().length();
			}
			if ( width > 32 ) {
				width = 32;
			}
			tcol.setPreferredWidth( width * 10 );
		}
	}

	class NumericCellRenderer extends DefaultTableCellRenderer
	{
		private static final long serialVersionUID = 1L;

		public Component getTableCellRendererComponent( JTable table,
			Object value, boolean isSelected, boolean hasFocus,
			int row, int column )
		{
			JLabel c = (JLabel )super.getTableCellRendererComponent( table,
			            value, isSelected, hasFocus, row, column );
			c.setHorizontalAlignment( RIGHT );
			return c;
		}
	}

	public static class Model extends DefaultTableModel
	{
		private static final long serialVersionUID = 1L;
	}

}
